from django.http import FileResponse, HttpResponse

from rest_framework import viewsets
from rest_framework.authentication import BasicAuthentication, SessionAuthentication, TokenAuthentication
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.parsers import FormParser, MultiPartParser
from rest_framework.response import Response

from daiquiri.core.responses import HttpResponseSeeOther
from daiquiri.core.utils import get_client_ip

from .filters import UWSFilterBackend
from .models import Job
from .renderers import UWSErrorRenderer, UWSRenderer
from .serializers import (
    AsyncJobSerializer,
    JobListSerializer,
    JobRetrieveSerializer,
    JobUpdateSerializer,
    SyncJobSerializer,
)
from .utils import get_content_type, get_job_results, get_job_url


class JobViewSet(viewsets.GenericViewSet):

    authentication_classes = (SessionAuthentication, BasicAuthentication, TokenAuthentication)
    parser_classes = (FormParser, MultiPartParser)

    parameter_map = {}

    def rewrite_exception(self, exception):
        detail = {}
        for field, field_errors in exception.detail.items():
            parameter = dict(map(reversed, self.parameter_map.items())).get(field, field.upper())
            detail[parameter] = field_errors
        return detail

    def handle_upload(self, job, upload_string):
        pass


class SyncJobViewSet(JobViewSet):

    serializer_class = SyncJobSerializer

    renderer_classes = (UWSErrorRenderer, )

    def list(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.GET)
        serializer.is_valid(raise_exception=True)

        return self.perform_sync_job(request, serializer.data)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        return self.perform_sync_job(request, serializer.validated_data)

    def perform_sync_job(self, request, data):
        # create the job objects
        job = self.get_queryset().model(
            job_type=Job.JOB_TYPE_SYNC,
            owner=(None if self.request.user.is_anonymous else self.request.user),
            response_format=data.get('RESPONSEFORMAT'),
            max_records=data.get('MAXREC'),
            run_id=data.get('RUNID'),
            client_ip=get_client_ip(self.request)
        )

        # add parameters to the job object
        for parameter, model_field in self.parameter_map.items():
            value = data.get(parameter)
            if value is not None:
                setattr(job, model_field, value)

        # handle possible uploads
        try:
            self.handle_upload(job, data.get('UPLOAD'))
        except ValueError as e:
            raise ValidationError('Could not parse VOTable') from e

        try:
            job.process()
        except ValidationError as e:
            raise ValidationError(self.rewrite_exception(e)) from e

        return FileResponse(job.run_sync(), content_type=job.formats[job.response_format])


class AsyncJobViewSet(JobViewSet):

    serializer_class = AsyncJobSerializer
    renderer_classes = (UWSErrorRenderer, )
    filter_backends = (UWSFilterBackend, )

    def get_success_url(self, job=None):
        if job:
            kwargs = {'pk': job.pk}
        else:
            kwargs = self.kwargs

        return get_job_url(self.request, kwargs=kwargs)

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        serializer = JobListSerializer(queryset, many=True)
        rendered_data = UWSRenderer().render(serializer.data, renderer_context=self.get_renderer_context())
        return HttpResponse(rendered_data, content_type=get_content_type(request, UWSRenderer))

    def retrieve(self, request, *args, **kwargs):
        job = self.get_object()
        serializer = JobRetrieveSerializer(job, context={'request': request})
        rendered_data = UWSRenderer().render(serializer.data, renderer_context=self.get_renderer_context())
        return HttpResponse(rendered_data, content_type=get_content_type(request, UWSRenderer))

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        # create the job objects
        job = self.get_queryset().model(
            job_type=Job.JOB_TYPE_ASYNC,
            owner=(None if self.request.user.is_anonymous else self.request.user),
            response_format=serializer.validated_data.get('RESPONSEFORMAT'),
            max_records=serializer.validated_data.get('MAXREC'),
            # uploads=handle_uploads(request, serializer.validated_data.get('UPLOAD'), self.get_upload_directory()),
            run_id=serializer.validated_data.get('RUNID'),
            client_ip=get_client_ip(self.request)
        )

        # add parameters to the job object
        for parameter, model_field in self.parameter_map.items():
            value = serializer.validated_data.get(parameter)
            if value is not None:
                setattr(job, model_field, value)

        # handle possible uploads
        self.handle_upload(job, serializer.validated_data.get('UPLOAD'))

        try:
            job.process()
        except ValidationError as e:
            raise ValidationError(self.rewrite_exception(e)) from e

        job.save()

        if serializer.validated_data.get('PHASE') == job.PHASE_RUN:
            job.run()

        return HttpResponseSeeOther(self.get_success_url(job))

    def update(self, request, *args, **kwargs):
        self.get_object()  # necessary to check permissions

        serializer = JobUpdateSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        if 'ACTION' in serializer.data:
            if serializer.data['ACTION'] == 'DELETE':
                return self.destroy(self, request)
            else:
                raise ValidationError({
                    'PHASE': 'Unsupported value.'
                })
        else:
            raise ValidationError({
                'ACTION': 'Parameter not found.'
            })

    def destroy(self, request, *args, **kwargs):
        job = self.get_object()
        job.archive()
        return HttpResponseSeeOther(self.get_success_url())

    @action(detail=True, methods=['get'])
    def results(self, request, pk, key=None):
        job = self.get_object()

        rendered_data = UWSRenderer().render({
            'results': get_job_results(request, job)
        }, renderer_context=self.get_renderer_context())
        return HttpResponse(rendered_data, content_type=get_content_type(request, UWSRenderer))

    @action(detail=True, methods=['get'], url_path=r'results/(?P<result>[A-Za-z0-9\-]+)', url_name='result')
    def result(self, request, pk, result):
        job = self.get_object()

        if result == 'result':
            return FileResponse(job.stream(job.response_format), content_type=job.formats[job.response_format])
        elif result in job.formats:
            return FileResponse(job.stream(result), content_type=job.formats[result])
        else:
            raise ValidationError({
                'result': 'Unsupported value.'
            })

    @action(detail=True, methods=['get'])
    def parameters(self, request, pk):
        rendered_data = UWSRenderer().render({
            'parameters': self.get_object().parameters
        }, renderer_context=self.get_renderer_context())
        return HttpResponse(rendered_data, content_type=get_content_type(request, UWSRenderer))

    @action(detail=True, methods=['get', 'post'])
    def destruction(self, request, pk):
        job = self.get_object()

        if request.method == 'GET':
            if job.destruction_time:
                # use the JobUpdateSerializer to create timestamp
                serializer = JobUpdateSerializer({
                    'DESTRUCTION': job.destruction_time
                })

                return HttpResponse(serializer.data['DESTRUCTION'])
            else:
                return HttpResponse()
        else:
            serializer = JobUpdateSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)

            if 'DESTRUCTION' in serializer.data:
                job.destruction_time = serializer.data['DESTRUCTION']
                job.save()
                return HttpResponseSeeOther(self.get_success_url(), status=303)
            else:
                raise ValidationError({
                    'DESTRUCTION': 'Parameter not found.'
                })

    @action(detail=True, methods=['get', 'post'])
    def executionduration(self, request, pk):
        job = self.get_object()

        if request.method == 'GET':
            return HttpResponse(job.execution_duration)
        else:
            serializer = JobUpdateSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)

            if 'EXECUTIONDURATION' in serializer.data:
                job.execution_duration = serializer.data['EXECUTIONDURATION']
                job.save()
                return HttpResponseSeeOther(self.get_success_url())
            else:
                raise ValidationError({
                    'EXECUTIONDURATION': 'Parameter not found.'
                })

    @action(detail=True, methods=['get', 'post'])
    def phase(self, request, pk):
        job = self.get_object()

        if request.method == 'GET':
            return HttpResponse(job.phase)
        else:
            serializer = JobUpdateSerializer(data=request.data)
            serializer.is_valid(raise_exception=True)

            if 'PHASE' in serializer.data:
                phase = serializer.data['PHASE']
                if phase == job.PHASE_RUN:
                    try:
                        job.process()
                    except ValidationError as e:
                        raise ValidationError(self.rewrite_exception(e)) from e

                    job.run()
                    return HttpResponseSeeOther(self.get_success_url())

                elif phase == job.PHASE_ABORT:
                    job.abort()
                    return HttpResponseSeeOther(self.get_success_url())

                else:
                    raise ValidationError({
                        'PHASE': 'Unsupported value.'
                    })
            else:
                raise ValidationError({
                    'PHASE': 'Parameter not found.'
                })

    @action(detail=True, methods=['get'])
    def error(self, request, pk):
        job = self.get_object()
        return Response(job.error_summary, content_type='application/xml') if job.error_summary else HttpResponse()

    @action(detail=True, methods=['get'])
    def quote(self, request, pk):
        job = self.get_object()
        return HttpResponse(job.quote) if job.quote else HttpResponse()

    @action(detail=True, methods=['get'])
    def owner(self, request, pk):
        job = self.get_object()
        return HttpResponse(job.owner) if job.owner else HttpResponse()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-22 08:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0022_table_size'),
    ]

    operations = [
        migrations.AlterField(
            model_name='column',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='column',
            name='index_for',
            field=models.CharField(blank=True, default='', help_text='The columns which this column is an index for (e.g. for pgSphere).', max_length=256, verbose_name='Index for'),
        ),
        migrations.AlterField(
            model_name='column',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='function',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='function',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='license',
            field=models.CharField(blank=True, choices=[('CC0', 'CC0 1.0 Universal (CC0 1.0)'), ('PD', 'Public Domain Mark'), ('BY', 'Attribution 4.0 International (CC BY 4.0)'), ('BY_SA', 'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), ('BY_ND', 'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), ('BY_NC', 'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), ('BY_NC_SA', 'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), ('BY_NC_ND', 'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='table',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Access level'),
        ),
        migrations.AlterField(
            model_name='table',
            name='license',
            field=models.CharField(blank=True, choices=[('CC0', 'CC0 1.0 Universal (CC0 1.0)'), ('PD', 'Public Domain Mark'), ('BY', 'Attribution 4.0 International (CC BY 4.0)'), ('BY_SA', 'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), ('BY_ND', 'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), ('BY_NC', 'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), ('BY_NC_SA', 'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), ('BY_NC_ND', 'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='table',
            name='metadata_access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Metadata access level'),
        ),
        migrations.AlterField(
            model_name='table',
            name='type',
            field=models.CharField(choices=[('table', 'Table'), ('view', 'View')], max_length=8, verbose_name='Type of table'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-13 11:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0020_blank_index_for'),
    ]

    operations = [
        migrations.AddField(
            model_name='table',
            name='nrows',
            field=models.BigIntegerField(blank=True, null=True, verbose_name='Number of rows in the table'),
        ),
    ]



import hashlib

from django.db import models


class JobManager(models.Manager):

    def filter_by_owner(self, user):
        if not user or user.is_anonymous:
            return self.get_queryset().filter(owner=None)
        else:
            return self.get_queryset().filter(owner=user)

    def get_active(self, user):
        # get the number of PENDING, QUEUED or EXECUTING jobs
        return self.filter_by_owner(user).filter(phase__in=self.model.PHASE_ACTIVE)

    def get_hash(self, user):
        jobs = self.filter_by_owner(user).exclude(phase__in=self.model.PHASE_ARCHIVED).values('id', 'phase')
        jobs_string = '\n'.join(['{id}-{phase}'.format(**job) for job in jobs])
        return hashlib.sha256(jobs_string.encode()).hexdigest()



from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand

from daiquiri.metadata.models import Column, Schema, Table
from daiquiri.tap.constants import TAP_SCHEMA_METADATA


class Command(BaseCommand):

    def handle(self, *args, **options):
        # remove the current tap schema
        try:
            Schema.objects.get(name=settings.TAP_SCHEMA).delete()
        except Schema.DoesNotExist:
            pass

        schema = Schema()
        schema.name = settings.TAP_SCHEMA
        schema.description = TAP_SCHEMA_METADATA['description']
        schema.order = TAP_SCHEMA_METADATA['order']
        schema.access_level = TAP_SCHEMA_METADATA['access_level']
        schema.metadata_access_level = TAP_SCHEMA_METADATA['metadata_access_level']
        schema.save()

        for table_metadata in TAP_SCHEMA_METADATA['tables']:
            table = Table()
            table.schema = schema
            table.name = table_metadata['name']
            table.description = table_metadata['description']
            table.order = table_metadata['order']
            table.access_level = table_metadata['access_level']
            table.metadata_access_level = table_metadata['metadata_access_level']
            table.save()

            for column_metadata in table_metadata['columns']:
                column = Column()
                column.table = table
                column.name = column_metadata['name']
                column.description = column_metadata['description']
                column.order = column_metadata['order']
                column.datatype = column_metadata['datatype']
                column.arraysize = column_metadata['arraysize']
                column.std = column_metadata['std']
                column.access_level = column_metadata['access_level']
                column.metadata_access_level = column_metadata['metadata_access_level']
                column.save()

        if apps.is_installed('daiquiri.datalink'):
            from daiquiri.datalink.constants import DATALINK_FIELDS, DATALINK_TABLE

            table = Table()
            table.schema = schema
            table.name = DATALINK_TABLE['name']
            table.description = DATALINK_TABLE['description']
            table.order = DATALINK_TABLE['order']
            table.access_level = DATALINK_TABLE['access_level']
            table.metadata_access_level = DATALINK_TABLE['metadata_access_level']
            table.save()

            for column_metadata in DATALINK_FIELDS:
                column = Column()
                column.table = table
                column.name = column_metadata['name']
                column.order = column_metadata['order']
                column.description = column_metadata['description']
                column.ucd = column_metadata['ucd']
                column.datatype = column_metadata['datatype']
                column.arraysize = column_metadata['arraysize']
                column.std = column_metadata['std']
                column.access_level = column_metadata['access_level']
                column.metadata_access_level = column_metadata['metadata_access_level']
                column.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-07 13:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0005_more_fields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='column',
            name='datatype',
            field=models.CharField(blank=True, help_text='The datatype of the column on the database server.', max_length=256, null=True, verbose_name='Datatype'),
        ),
        migrations.AlterField(
            model_name='column',
            name='description',
            field=models.TextField(blank=True, help_text='A brief description of the column to be displayed in the user interface.', null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='column',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups which have access to the column.', to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='column',
            name='indexed',
            field=models.BooleanField(default=False, help_text='Designates whether the column is indexed.', verbose_name='Indexed'),
        ),
        migrations.AlterField(
            model_name='column',
            name='name',
            field=models.CharField(help_text='Identifier of the column on the database server.', max_length=256, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='column',
            name='order',
            field=models.IntegerField(blank=True, help_text='Position in lists.', null=True, verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='column',
            name='principal',
            field=models.BooleanField(default=False, help_text='Designates whether the column is considered a core part of the content.', verbose_name='Principal'),
        ),
        migrations.AlterField(
            model_name='column',
            name='size',
            field=models.IntegerField(blank=True, help_text='The length of variable length datatypes, e.g. varchar(256).', null=True, verbose_name='Size'),
        ),
        migrations.AlterField(
            model_name='column',
            name='std',
            field=models.BooleanField(default=False, help_text='Designates whether the column is defined by some standard.', verbose_name='Standard'),
        ),
        migrations.AlterField(
            model_name='column',
            name='table',
            field=models.ForeignKey(help_text='Table the column belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='columns', to='daiquiri_metadata.Table'),
        ),
        migrations.AlterField(
            model_name='column',
            name='ucd',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='IVOA UCDs'),
        ),
        migrations.AlterField(
            model_name='column',
            name='unit',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='Unit'),
        ),
        migrations.AlterField(
            model_name='column',
            name='utype',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='IVOA Utype'),
        ),
        migrations.AlterField(
            model_name='database',
            name='attribution',
            field=models.TextField(blank=True, help_text='The desired attribution for the database.', null=True, verbose_name='Attribution'),
        ),
        migrations.AlterField(
            model_name='database',
            name='description',
            field=models.TextField(blank=True, help_text='A brief description of the database to be displayed in the user interface.', null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='database',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups which have access to the database.', to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='database',
            name='license',
            field=models.CharField(blank=True, choices=[(b'CC0', b'CC0 1.0 Universal (CC0 1.0)'), (b'PD', b'Public Domain Mark'), (b'BY', b'Attribution 4.0 International (CC BY 4.0)'), (b'BY_SA', b'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), (b'BY_ND', b'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), (b'BY_NC', b'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), (b'BY_NC_SA', b'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), (b'BY_NC_ND', b'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='database',
            name='long_description',
            field=models.TextField(blank=True, help_text='A more extensive description of the database to be displayed on the public database page.', null=True, verbose_name='Long description'),
        ),
        migrations.AlterField(
            model_name='database',
            name='name',
            field=models.CharField(help_text='Name of the database on the database server.', max_length=256, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='database',
            name='order',
            field=models.IntegerField(blank=True, default=0, help_text='Position in lists.', null=True, verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='database',
            name='pid',
            field=models.URLField(blank=True, max_length=256, null=True, verbose_name='Persistent identifier'),
        ),
        migrations.AlterField(
            model_name='database',
            name='title',
            field=models.CharField(blank=True, help_text='Human readable title of the database.', max_length=256, null=True, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='database',
            name='utype',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='IVOA Utype'),
        ),
        migrations.AlterField(
            model_name='function',
            name='description',
            field=models.TextField(blank=True, help_text='A brief description of the function to be displayed in the user interface.', null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='function',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups which have access to this function.', to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='function',
            name='name',
            field=models.CharField(help_text='Identifier of the function on the server.', max_length=256, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='function',
            name='order',
            field=models.IntegerField(blank=True, help_text='Position in lists.', null=True, verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='function',
            name='query_string',
            field=models.CharField(help_text='Prototype of this function in a SQL query.', max_length=256),
        ),
        migrations.AlterField(
            model_name='table',
            name='attribution',
            field=models.TextField(blank=True, help_text='The desired attribution for the table.', null=True, verbose_name='Attribution'),
        ),
        migrations.AlterField(
            model_name='table',
            name='database',
            field=models.ForeignKey(help_text='Database the table belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='tables', to='daiquiri_metadata.Database', verbose_name='Database'),
        ),
        migrations.AlterField(
            model_name='table',
            name='description',
            field=models.TextField(blank=True, help_text='A brief description of the table to be displayed in the user interface.', null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='table',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups which have access to the table.', to='auth.Group', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='table',
            name='license',
            field=models.CharField(blank=True, choices=[(b'CC0', b'CC0 1.0 Universal (CC0 1.0)'), (b'PD', b'Public Domain Mark'), (b'BY', b'Attribution 4.0 International (CC BY 4.0)'), (b'BY_SA', b'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), (b'BY_ND', b'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), (b'BY_NC', b'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), (b'BY_NC_SA', b'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), (b'BY_NC_ND', b'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='table',
            name='long_description',
            field=models.TextField(blank=True, help_text='A more extensive description of the table to be displayed on the public database page.', null=True, verbose_name='Long description'),
        ),
        migrations.AlterField(
            model_name='table',
            name='name',
            field=models.CharField(help_text='Identifier of the table on the database server.', max_length=256, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='table',
            name='order',
            field=models.IntegerField(blank=True, help_text='Position in lists.', null=True, verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='table',
            name='pid',
            field=models.URLField(blank=True, max_length=256, null=True, verbose_name='Persistent identifier'),
        ),
        migrations.AlterField(
            model_name='table',
            name='title',
            field=models.CharField(blank=True, help_text='Human readable title of the table.', max_length=256, null=True, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='table',
            name='type',
            field=models.CharField(choices=[(b'table', 'Table'), (b'view', 'View')], max_length=8, verbose_name='Type of table'),
        ),
        migrations.AlterField(
            model_name='table',
            name='utype',
            field=models.CharField(blank=True, max_length=256, null=True, verbose_name='IVOA Utype'),
        ),
    ]



from django.dispatch import Signal

user_created = Signal()
user_updated = Signal()
user_deleted = Signal()
user_groups_updated = Signal()

user_confirmed = Signal()
user_rejected = Signal()
user_activated = Signal()
user_disabled = Signal()
user_enabled = Signal()



from rest_framework import serializers

from daiquiri.core.constants import ACCESS_LEVEL_PUBLIC
from daiquiri.metadata.models import Column, Schema, Table


class ColumnSerializer(serializers.ModelSerializer):

    class Meta:
        model = Column
        fields = (
            'name',
            'description',
            'datatype',
            'ucd',
            'unit',
            'indexed',
            'principal',
            'std'
        )


class TableSerializer(serializers.ModelSerializer):

    name = serializers.SerializerMethodField()
    columns = ColumnSerializer(many=True)

    class Meta:
        model = Table
        fields = (
            'name',
            'description',
            'columns',
            'doi',
            'nrows'
        )

    def get_name(self, obj):
        return str(obj)


class SchemaSerializer(serializers.ModelSerializer):

    tables = serializers.SerializerMethodField()

    class Meta:
        model = Schema
        fields = (
            'name',
            'description',
            'tables'
        )

    def get_tables(self, obj):
        # filter the tables which are published for the groups of the user
        queryset = obj.tables.filter(metadata_access_level=ACCESS_LEVEL_PUBLIC, published__isnull=False)
        return TableSerializer(queryset, many=True).data



from django import template
from django.conf import settings
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from daiquiri.core.utils import get_doi_url

from ..models import Schema

register = template.Library()


@register.inclusion_tag('metadata/tags/schemas_menu.html', takes_context=True)
def schemas_menu(context, tables=True):

    schemas = Schema.objects.filter_by_metadata_access_level(context.request.user)

    context['schemas'] = []
    for schema in schemas:
        context['schemas'].append({
            'name': schema.name,
            'label': schema.title or schema.name,
            'tables': [{
                'name': table.name,
                'label': f'{schema.name}.{table.name}'
                } for table in schema.tables.filter_by_metadata_access_level(context.request.user)
            ] if tables else []
        })

    return context


@register.inclusion_tag('metadata/tags/access_panel.html')
def access_panel(doi, dataset=None):
    return {
        'dataset': dataset or _('dataset')
    }


@register.simple_tag()
def doi_link(doi):
    url = get_doi_url(doi)
    return format_html('<a class="break" href="{}">{}</a>', url, url)


@register.inclusion_tag('metadata/tags/doi_panel.html')
def doi_panel(doi, dataset=None):
    return {
        'doi_url': get_doi_url(doi),
        'dataset': dataset or _('dataset')
    }


@register.inclusion_tag('metadata/tags/license_panel.html')
def license_panel(license, dataset=None):
    return {
        'license': license,
        'license_url': settings.LICENSE_URLS[license],
        'license_label': dict(settings.LICENSE_CHOICES)[license],
        'dataset': dataset or _('dataset')
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-04-18 14:10
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0017_big_integer_fields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queryjob',
            name='metadata',
            field=jsonfield.fields.JSONField(blank=True),
        ),
    ]



# Generated by Django 2.1.4 on 2019-05-29 11:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0024_django2'),
    ]

    operations = [
        migrations.AddField(
            model_name='schema',
            name='published',
            field=models.DateField(blank=True, null=True, verbose_name='Published'),
        ),
        migrations.AddField(
            model_name='schema',
            name='updated',
            field=models.DateField(blank=True, null=True, verbose_name='Updated'),
        ),
        migrations.AddField(
            model_name='table',
            name='published',
            field=models.DateField(blank=True, null=True, verbose_name='Published'),
        ),
        migrations.AddField(
            model_name='table',
            name='updated',
            field=models.DateField(blank=True, null=True, verbose_name='Updated'),
        ),
    ]



from django import forms
from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

from daiquiri.core.forms import HoneypotField
from daiquiri.core.utils import get_detail_fields, sanitize_str

from .models import Profile


class UserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('first_name', 'last_name')
        widgets = {
            'first_name': forms.TextInput(attrs={'placeholder': _('First name')}),
            'last_name': forms.TextInput(attrs={'placeholder': _('Last name')}),
        }


class ProfileForm(forms.ModelForm):

    class Meta:
        model = Profile
        fields = ()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for key, field in get_detail_fields(settings.AUTH_DETAIL_KEYS):
            if self.instance.details and key in self.instance.details:
                field.initial = self.instance.details[key]

            self.fields[key] = field

    def save(self, *args, **kwargs):
        # create an empty details dict if it does not exist
        if not self.instance.details:
            self.instance.details = {}

        # store the form date for each detail key
        for detail_key in settings.AUTH_DETAIL_KEYS:
            self.instance.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        return super().save(*args, **kwargs)


class SignupForm(ProfileForm):

    first_name = forms.CharField(max_length=30,
        label=_('First name'), widget=forms.TextInput(attrs={'placeholder': _('First name')}))
    last_name = forms.CharField(max_length=30,
        label=_('Last name'), widget=forms.TextInput(attrs={'placeholder': _('Last name')}))

    field_order = ['username', 'email', 'password1', 'password2']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # add a consent field, the label is added in the template
        if settings.AUTH_TERMS_OF_USE:
            tos_url = reverse('terms_of_use')
            self.fields['consent'] = forms.BooleanField()
            self.fields['consent'].label = mark_safe(f'I agree to the <a href="{tos_url}" data-bs-toggle="modal" '
                                                      'data-bs-target="#terms-of-use-modal">terms of use</a>.')
        # add honeypot field
        if settings.HONEYPOT_ENABLED is True:
            self.fields[sanitize_str(settings.HONEYPOT_FIELD_NAME)] = HoneypotField()

    def clean(self):
        if settings.AUTH_TERMS_OF_USE and not self.cleaned_data.get('consent'):
            raise ValidationError(_('You need to agree to the terms of use.'))

    def signup(self, request, user):
        # create an empty details dict
        user.profile.details = {}

        # store the form date for each detail key
        for detail_key in settings.AUTH_DETAIL_KEYS:
            user.profile.details[detail_key['key']] = self.cleaned_data[detail_key['key']]

        # store the consent field
        if settings.AUTH_TERMS_OF_USE:
            user.profile.consent = self.cleaned_data['consent']

        # save the profile model
        user.profile.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-13 11:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0016_rename_database_to_schema'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queryjob',
            name='nrows',
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='size',
            field=models.BigIntegerField(blank=True, null=True),
        ),
    ]



import subprocess
import sys

from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):
        call_command('npm', 'ci')
        call_command('npm', 'run', 'build:prod')
        subprocess.call(['/bin/bash', '-c', f'{sys.executable} -m build'])



import logging
import os
import zipfile

from django.conf import settings
from django.db.utils import InternalError, OperationalError, ProgrammingError
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from celery import shared_task

from daiquiri.core.tasks import Task
from daiquiri.stats.models import Record


class RunQueryTask(Task):

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        super().on_failure(exc, task_id, args, kwargs, einfo)

        # always import daiquiri packages inside the task
        from daiquiri.query.models import QueryJob

        # get logger
        logger = logging.getLogger(__name__)

        # get job_id from the original task args
        job_id = args[0]

        # log raised exception
        logger.error('run_query %s raised an exception (%s)', job_id, exc)

        # set phase and error_summary of the crashed job
        job = QueryJob.objects.get(pk=job_id)
        job.phase = job.PHASE_ERROR
        job.error_summary = str(_('There has been an server error with your job.'))
        job.save()


@shared_task(base=RunQueryTask)
def run_database_query_task(job_id):
    # always import daiquiri packages inside the task
    from daiquiri.core.adapter import DatabaseAdapter
    from daiquiri.query.models import QueryJob
    from daiquiri.query.utils import get_job_columns, get_job_sources, get_quota, ingest_uploads
    from daiquiri.stats.models import Record

    # get logger
    logger = logging.getLogger(__name__)

    # get the job object from the database
    job = QueryJob.objects.get(pk=job_id)

    if job.phase == job.PHASE_QUEUED:
        # get the adapter with the database specific functions
        adapter = DatabaseAdapter()

        # create the database of the user if it not already exists
        try:
            adapter.create_user_schema_if_not_exists(job.schema_name)
        except OperationalError as e:
            job.phase = job.PHASE_ERROR
            job.error_summary = str(e)
            job.save()

            return job.phase

        # check if the quota is exceeded
        if QueryJob.objects.get_size(job.owner) > get_quota(job.owner):
            job.phase = job.PHASE_ERROR
            job.error_summary = str(_('Quota is exceeded. Please remove some of your jobs.'))
            job.save()

            return job.phase

        # set database and start time
        job.pid = adapter.fetch_pid()
        job.actual_query = adapter.build_query(
            job.schema_name,
            job.table_name,
            job.native_query,
            job.timeout,
            job.max_records
        )
        job.phase = job.PHASE_EXECUTING
        job.start_time = now()
        job.save()

        logger.info('job %s started', job.id)

        # get the actual query and submit the job to the database
        try:
            job.metadata['upload_columns'] = ingest_uploads(job.uploads, job.owner)

            # this is where the work is done (and the time is spend)
            adapter.submit_query(job.actual_query)

        except (ProgrammingError, InternalError, ValueError) as e:
            job.phase = job.PHASE_ERROR
            job.error_summary = str(e)
            logger.info('job %s failed (%s)', job.id, job.error_summary)

        except OperationalError as e:
            # load the job again and check if the job was killed
            job = QueryJob.objects.get(pk=job_id)

            if job.phase != job.PHASE_ABORTED:
                job.phase = job.PHASE_ERROR
                job.error_summary = str(e)
                logger.info('job %s failed (%s)', job.id, job.error_summary)

        else:
            # get additional information about the completed job
            job.phase = job.PHASE_COMPLETED
            logger.info('job %s completed', job.id)

        finally:
            # get timing and save the job object
            job.end_time = now()

            # get additional information about the completed job
            if job.phase == job.PHASE_COMPLETED:
                job.nrows = adapter.count_rows(job.schema_name, job.table_name)
                job.size = adapter.fetch_size(job.schema_name, job.table_name)

                # fetch the metadata for used tables
                job.metadata['sources'] = get_job_sources(job)

                # fetch the metadata for the columns and fetch additional metadata from the metadata store
                job.metadata['columns'] = get_job_columns(job)

            # remove unneeded metadata
            job.metadata.pop('display_columns', None)
            job.metadata.pop('tables', None)
            job.metadata.pop('upload_columns', None)
            job.metadata.pop('user_columns', None)

            # create a stats record for this job
            Record.objects.create(
                time=job.end_time,
                resource_type='QUERY',
                resource={
                    'job_id': job.id,
                    'job_type': job.job_type,
                },
                client_ip=job.client_ip,
                user=job.owner,
                size=job.size
            )

            job.save()

    return job.phase


@shared_task(base=Task)
def run_database_ingest_task(job_id, file_path):
    from daiquiri.core.adapter import DatabaseAdapter
    from daiquiri.query.models import QueryJob
    from daiquiri.query.utils import get_quota, ingest_table
    from daiquiri.stats.models import Record

    # get logger
    logger = logging.getLogger(__name__)

    # get the job object from the database
    job = QueryJob.objects.get(pk=job_id)

    if job.phase == job.PHASE_QUEUED:
        # get the adapter with the database specific functions
        adapter = DatabaseAdapter()

        # create the database of the user if it not already exists
        try:
            adapter.create_user_schema_if_not_exists(job.schema_name)
        except OperationalError as e:
            job.phase = job.PHASE_ERROR
            job.error_summary = str(e)
            job.save()

            return job.phase

        # check if the quota is exceeded
        if QueryJob.objects.get_size(job.owner) > get_quota(job.owner):
            job.phase = job.PHASE_ERROR
            job.error_summary = str(_('Quota is exceeded. Please remove some of your jobs.'))
            job.save()

            return job.phase

        # set database and start time
        job.pid = adapter.fetch_pid()
        job.phase = job.PHASE_EXECUTING
        job.start_time = now()
        job.save()

        logger.info('job %s started', job.id)

        # create the table and insert the data
        try:
            columns = ingest_table(job.schema_name, job.table_name, file_path)

        except (ProgrammingError, InternalError, ValueError) as e:
            job.phase = job.PHASE_ERROR
            job.error_summary = str(e)
            logger.info('job %s failed (%s)', job.id, job.error_summary)

        except OperationalError as e:
            # load the job again and check if the job was killed
            job = QueryJob.objects.get(pk=job_id)

            if job.phase != job.PHASE_ABORTED:
                job.phase = job.PHASE_ERROR
                job.error_summary = str(e)
                logger.info('job %s failed (%s)', job.id, job.error_summary)

        else:
            # get additional information about the completed job
            job.phase = job.PHASE_COMPLETED
            logger.info('job %s completed', job.id)

        finally:
            # get timing and save the job object
            job.end_time = now()

            # get additional information about the completed job
            if job.phase == job.PHASE_COMPLETED:
                job.nrows = adapter.count_rows(job.schema_name, job.table_name)
                job.size = adapter.fetch_size(job.schema_name, job.table_name)

                # store the metadata for the columns from the VOTable
                job.metadata = {
                    'columns': columns
                }

            # create a stats record for this job
            Record.objects.create(
                time=job.end_time,
                resource_type='UPLOAD',
                resource={
                    'job_id': job.id,
                    'job_type': job.job_type,
                },
                client_ip=job.client_ip,
                user=job.owner,
                size=job.size
            )

            job.save()

    return job.phase


@shared_task(base=Task)
def create_download_table_task(download_id):
    # always import daiquiri packages inside the task
    from daiquiri.query.models import DownloadJob

    # get logger
    logger = logging.getLogger(__name__)

    # get the job object from the database
    download_job = DownloadJob.objects.get(pk=download_id)

    if download_job.phase == download_job.PHASE_QUEUED:
        # log start
        logger.info('download_job %s started', download_job.file_path)

        # create directory if necessary
        try:
            os.mkdir(os.path.dirname(download_job.file_path))
        except OSError:
            pass

        download_job.phase = download_job.PHASE_EXECUTING
        download_job.start_time = now()
        download_job.save()

        # write file using the generator in the adapter
        if download_job.format_key == 'fits':
            write_label = 'wb'
        else:
            write_label = 'w'
        try:

            with open(download_job.file_path, write_label) as f:
                for line in download_job.query_job.stream(download_job.format_key):
                    f.write(line)

        except Exception as e:
            download_job.phase = download_job.PHASE_ERROR
            download_job.error_summary = str(e)
            download_job.save()
            logger.info('download_job %s failed (%s)', download_job.id, download_job.error_summary)

            raise e
        else:
            download_job.phase = download_job.PHASE_COMPLETED
            logger.info('download_job %s completed', download_job.file_path)
            Record.objects.create(
                time=download_job.start_time,
                resource_type='CREATE_FILE',
                resource={
                    'job_id': download_job.id,
                    'job_type': download_job.job_type,
                    'file_path': download_job.file_path
                },
                client_ip=download_job.client_ip,
                size=os.path.getsize(download_job.file_path)
            )
        finally:
            download_job.end_time = now()
            download_job.save()


@shared_task(track_started=True, base=Task)
def create_download_archive_task(archive_id):
    # always import daiquiri packages inside the task
    from daiquiri.query.models import QueryArchiveJob

    # get logger
    logger = logging.getLogger(__name__)

    # get the job object from the database
    archive_job = QueryArchiveJob.objects.get(pk=archive_id)

    if archive_job.phase == archive_job.PHASE_QUEUED:
        # log start
        logger.info('create_archive_zip_file %s started', archive_job.file_path)

        # create directory if necessary
        try:
            os.makedirs(os.path.dirname(archive_job.file_path))
        except OSError:
            pass

        archive_job.phase = archive_job.PHASE_EXECUTING
        archive_job.start_time = now()
        archive_job.save()

        # create a zipfile with all files
        try:
            with zipfile.ZipFile(archive_job.file_path, 'w') as z:
                os.chdir(settings.FILES_BASE_PATH)
                for file_path in archive_job.files:
                    z.write(file_path)

        except Exception as e:
            archive_job.phase = archive_job.PHASE_ERROR
            archive_job.error_summary = str(e)
            archive_job.save()
            logger.info('archive_job %s failed (%s)', archive_job.id, archive_job.error_summary)
            raise e

        archive_job.end_time = now()
        archive_job.phase = archive_job.PHASE_COMPLETED
        archive_job.save()
        Record.objects.create(
            time=archive_job.start_time,
            resource_type='CREATE_ZIP',
            resource={
                'job_id': archive_job.id,
                'job_type': archive_job.job_type,
                'file_path': archive_job.file_path
            },
            client_ip=archive_job.client_ip,
            size=os.path.getsize(archive_job.file_path)
        )

        # log completion
        logger.info('create_archive_zip_file %s completed', archive_job.file_path)


@shared_task(base=Task)
def rename_database_table_task(schema_name, table_name, new_table_name):
    from daiquiri.core.adapter import DatabaseAdapter

    DatabaseAdapter().rename_table(schema_name, table_name, new_table_name)


@shared_task(base=Task)
def drop_database_table_task(schema_name, table_name):
    from daiquiri.core.adapter import DatabaseAdapter

    # drop the corresponding database table, but fail silently
    try:
        DatabaseAdapter().drop_table(schema_name, table_name)
    except ProgrammingError:
        pass


@shared_task(base=Task)
def abort_database_query_task(pid):
    from daiquiri.core.adapter import DatabaseAdapter

    # abort the job on the database
    try:
        DatabaseAdapter().abort_query(pid)
    except OperationalError:
        # the query was probably killed before
        pass



from __future__ import unicode_literals

from django.db import migrations


def run_data_migration(apps, schema_editor):
    QueryJob = apps.get_model('daiquiri_query', 'QueryJob')
    Record = apps.get_model('daiquiri_stats', 'Record')

    for record in Record.objects.all():
        if record.resource_type == 'QUERY_JOB':
            try:
                job = QueryJob.objects.get(pk=record.resource['job_id'])
                record.resource['query'] = job.query
                record.resource['query_language'] = job.query_language

            except QueryJob.DoesNotExist:
                record.resource['job_id'] = None
                record.resource['query'] = None
                record.resource['query_language'] = None

            record.resource_type = 'QUERY'
            record.save()


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_stats', '0002_data_migration'),
        ('daiquiri_query', '0013_refactoring'),
    ]

    operations = [
        migrations.RunPython(run_data_migration)
    ]



from django.core.management.base import BaseCommand

from daiquiri.core.constants import GROUPS
from daiquiri.core.utils import setup_group


class Command(BaseCommand):

    def handle(self, *args, **options):

        for name in GROUPS:
            group, created = setup_group(name)

            if created:
                print (f'Group "{name}" created')
            else:
                print (f'Group "{name}" updated')



from django.conf import settings
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.query.models import QueryJob


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--user', help='Only scrub jobs for this user.')
        parser.add_argument('--delete', action='store_true', help='Delete stale tables.')

    def handle(self, *args, **options):
        if options['user']:
            usernames = [options['user']]
        else:
            usernames = ['anonymous', *list(User.objects.values_list('username', flat=True))]

        adapter = DatabaseAdapter()

        stale_tables = []
        for username in usernames:
            schema_name = settings.QUERY_USER_SCHEMA_PREFIX + username

            tables = adapter.fetch_tables(schema_name)
            QueryJob.objects.filter(schema_name=schema_name)

            for table in tables:
                job = QueryJob.objects.filter(
                    schema_name=schema_name,
                    table_name=table['name']
                ).first()

                if job and job.phase not in [QueryJob.PHASE_EXECUTING, QueryJob.PHASE_COMPLETED]:
                    stale_tables.append((schema_name, table['name'], job.phase if job else None))

        if stale_tables:
            print('The following database tables have no associated QueryJob:')

            for stale_table in stale_tables:
                print('{}.{} -> {}'.format(*stale_table))

            if options['delete']:
                for schema_name, table_name, phase in stale_tables:
                    adapter.drop_table(schema_name, table_name)

                print('The tables have been deleted.')
        else:
            print('No tables without associated QueryJob have been found.')



from django.core.management.base import BaseCommand

from daiquiri.metadata.models import Schema
from daiquiri.tap.models import (
    Column as TapColumn,
)
from daiquiri.tap.models import (
    Schema as TapSchema,
)
from daiquiri.tap.models import (
    Table as TapTable,
)
from daiquiri.tap.utils import update_schema


class Command(BaseCommand):

    def handle(self, *args, **options):

        TapSchema.objects.all().delete()
        TapTable.objects.all().delete()
        TapColumn.objects.all().delete()

        for schema in Schema.objects.all():
            update_schema(schema)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-18 09:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0010_queue_can_be_null'),
    ]

    operations = [
        migrations.AddField(
            model_name='queryjob',
            name='native_query',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.contrib import admin

from .models import Directory


class DirectoryAdmin(admin.ModelAdmin):
    search_fields = ('path', 'access_level')
    list_display = ('absolute_path', 'path' , 'depth', 'access_level')
    list_display_links = ('absolute_path', )
    readonly_fields = ('depth', )

admin.site.register(Directory, DirectoryAdmin)



from rest_framework import serializers

from ..models import Column, Function, Schema, Table


class ColumnSerializer(serializers.ModelSerializer):

    class Meta:
        model = Column
        fields = ('id', 'name')


class TableSerializer(serializers.ModelSerializer):

    columns = ColumnSerializer(many=True, read_only=True)

    class Meta:
        model = Table
        fields = ('id', 'name', 'columns')


class SchemaSerializer(serializers.ModelSerializer):

    tables = TableSerializer(many=True, read_only=True)

    class Meta:
        model = Schema
        fields = ('id', 'name', 'tables')


class FunctionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Function
        fields = ('id', 'name')



from django.http import Http404
from django.views.generic import TemplateView

from daiquiri.core.utils import get_model_field_meta
from daiquiri.core.views import CSRFViewMixin, ModelPermissionMixin

from .models import Column, Function, Schema, Table


class ManagementView(ModelPermissionMixin, CSRFViewMixin, TemplateView):
    template_name = 'metadata/management.html'
    permission_required = 'daiquiri_metadata.view_schema'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['meta'] = {
            'Schema': get_model_field_meta(Schema),
            'Table': get_model_field_meta(Table),
            'Column': get_model_field_meta(Column),
            'Function': get_model_field_meta(Function),
        }
        return context


class SchemasView(TemplateView):
    template_name = 'metadata/schemas.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        try:
            schemas = Schema.objects.filter_by_metadata_access_level(self.request.user).all()
        except Schema.DoesNotExist as e:
            raise Http404() from e

        context['schemas'] = schemas
        return context


class SchemaView(TemplateView):
    template_name = 'metadata/schema.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        schema_name = self.kwargs.get('schema_name')

        try:
            schema = Schema.objects.filter_by_metadata_access_level(self.request.user).get(name=schema_name)
        except Schema.DoesNotExist as e:
            raise Http404() from e

        context['schema'] = schema
        context['tables'] = schema.tables.filter_by_metadata_access_level(self.request.user)
        return context


class TableView(TemplateView):
    template_name = 'metadata/table.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        schema_name = self.kwargs.get('schema_name')
        table_name = self.kwargs.get('table_name')

        try:
            schema = Schema.objects.filter_by_metadata_access_level(self.request.user).get(name=schema_name)
        except Schema.DoesNotExist as e:
            raise Http404() from e

        try:
            table = Table.objects.filter_by_metadata_access_level(self.request.user) \
                                 .filter(schema=schema).get(name=table_name)
        except Table.DoesNotExist as e:
            raise Http404() from e

        context['schema'] = schema
        context['table'] = table
        context['columns'] = table.columns.filter_by_metadata_access_level(self.request.user)
        return context



# Generated by Django 3.2.10 on 2022-01-22 13:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0030_move_licenses_to_settings'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='column',
            options={'ordering': ('table__schema__order', 'table__order', 'order', 'name'), 'verbose_name': 'Column', 'verbose_name_plural': 'Columns'},
        ),
        migrations.AlterModelOptions(
            name='function',
            options={'ordering': ('order', 'name'), 'verbose_name': 'Function', 'verbose_name_plural': 'Functions'},
        ),
        migrations.AlterModelOptions(
            name='schema',
            options={'ordering': ('order', 'name'), 'verbose_name': 'Schema', 'verbose_name_plural': 'Schemas'},
        ),
        migrations.AlterModelOptions(
            name='table',
            options={'ordering': ('schema__order', 'order', 'name'), 'verbose_name': 'Table', 'verbose_name_plural': 'Tables'},
        ),
    ]



from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import Http404, HttpResponseRedirect, JsonResponse
from django.shortcuts import render
from django.urls import reverse_lazy
from django.views.generic import TemplateView

from rest_framework.authtoken.models import Token

from allauth.account.views import PasswordChangeView as AllauthPasswordChangeView
from allauth.account.views import PasswordSetView as AllauthPasswordSetView
from allauth.account.views import logout as allauth_logout

from daiquiri.auth.models import Profile
from daiquiri.core.utils import get_next, get_referer_path_info
from daiquiri.core.views import CSRFViewMixin, ModelPermissionMixin, StoreIdViewMixin

from .forms import ProfileForm, UserForm


@login_required()
def profile_update(request):
    user_form = UserForm(request.POST or None, instance=request.user)
    profile_form = ProfileForm(request.POST or None, instance=request.user.profile)

    if request.method == 'POST':
        if 'cancel' in request.POST:
            return HttpResponseRedirect(get_next(request))

        if user_form.is_valid() and profile_form.is_valid():
            user_form.save()
            profile_form.save()
            return HttpResponseRedirect(get_next(request))

    return render(request, 'account/account_profile.html', {
        'user_form': user_form,
        'profile_form': profile_form,
        'next': get_referer_path_info(request, default='/')
    })


def terms_of_use(request):
    context = {}
    if not settings.AUTH_TERMS_OF_USE:
        raise Http404

    if request.user.is_authenticated:
        profile = Profile.objects.get(user=request.user)
        if request.method == 'POST':
            if 'consent' in request.POST:
                profile.consent = True
                profile.save()
        context["consent"] = profile.consent

    return render(request, 'account/terms_of_use_page.html', context)


@login_required()
def profile_json(request):
    return JsonResponse({
        'username': request.user.username
    })


@login_required()
def token(request):
    if request.method == 'POST':
        try:
            Token.objects.get(user=request.user).delete()
        except Token.DoesNotExist:
            pass

    token, created = Token.objects.get_or_create(user=request.user)
    return render(request, 'account/account_token.html', {
        'token': token
    })


def logout(request, *args, **kwargs):
    response = allauth_logout(request, *args, **kwargs)

    return response


class UsersView(ModelPermissionMixin, CSRFViewMixin, StoreIdViewMixin, TemplateView):
    template_name = 'auth/users.html'
    permission_required = 'daiquiri_auth.view_profile'


class PasswordChangeView(AllauthPasswordChangeView):

    success_url = reverse_lazy('account_change_password_done')


password_change = login_required(PasswordChangeView.as_view())


class PasswordSetView(AllauthPasswordSetView):

    success_url = reverse_lazy('account_set_password_done')


password_set = login_required(PasswordSetView.as_view())



# Generated by Django 3.2.11 on 2022-01-28 12:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_oai', '0004_alter_record_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='resource_id',
            field=models.BigIntegerField(db_index=True, null=True, verbose_name='Resource id'),
        ),
    ]



from daiquiri.core.renderers import ErrorRenderer


class CutOutErrorRenderer(ErrorRenderer):

    def render_votable(self, data, accepted_media_type=None, renderer_context=None):
        self.start('INFO', {
            'ID': 'Error',
            'name': 'Error',
            'value': self.get_error_string(data)
        })
        self.end('INFO')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-05 10:19
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Column',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, null=True)),
                ('name', models.CharField(max_length=256)),
                ('description', models.TextField(blank=True, null=True)),
                ('unit', models.CharField(blank=True, max_length=256, null=True)),
                ('ucd', models.CharField(blank=True, max_length=256, null=True)),
                ('utype', models.CharField(blank=True, max_length=256, null=True)),
                ('datatype', models.CharField(blank=True, max_length=256, null=True)),
                ('size', models.IntegerField(blank=True, help_text='The length of variable length datatypes, e.g. varchar(256).', null=True)),
                ('principal', models.BooleanField(default=False, help_text='This column is considered a core part of the content.')),
                ('indexed', models.BooleanField(default=False, help_text='This column is indexed.')),
                ('std', models.BooleanField(default=False, help_text='This column is defined by some standard.')),
            ],
            options={
                'ordering': ('table__database__order', 'table__order', 'order'),
                'permissions': (('view_column', 'Can view Column'),),
                'verbose_name_plural': 'Columns',
                'verbose_name': 'Column',
            },
        ),
        migrations.CreateModel(
            name='Database',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, null=True)),
                ('name', models.CharField(max_length=256)),
                ('description', models.TextField(blank=True, null=True)),
                ('utype', models.CharField(blank=True, max_length=256, null=True)),
            ],
            options={
                'ordering': ('order',),
                'permissions': (('view_database', 'Can view Database'),),
                'verbose_name_plural': 'Databases',
                'verbose_name': 'Database',
            },
        ),
        migrations.CreateModel(
            name='Function',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, null=True)),
                ('name', models.CharField(max_length=256)),
                ('description', models.TextField(blank=True, null=True)),
            ],
            options={
                'ordering': ('order',),
                'permissions': (('view_function', 'Can view Function'),),
                'verbose_name_plural': 'Functions',
                'verbose_name': 'Function',
            },
        ),
        migrations.CreateModel(
            name='Table',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, null=True)),
                ('name', models.CharField(max_length=256)),
                ('description', models.TextField(blank=True, null=True)),
                ('type', models.CharField(choices=[('table', 'Table'), ('view', 'View')], max_length=8)),
                ('utype', models.CharField(blank=True, max_length=256, null=True)),
                ('database', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tables', to='daiquiri_metadata.Database')),
            ],
            options={
                'ordering': ('database__order', 'order'),
                'permissions': (('view_table', 'Can view Table'),),
                'verbose_name_plural': 'Tables',
                'verbose_name': 'Table',
            },
        ),
        migrations.AddField(
            model_name='column',
            name='table',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='columns', to='daiquiri_metadata.Table'),
        ),
    ]



from rest_framework.exceptions import ValidationError


class PersonListValidator:

    def __call__(self, persons):

        for person in persons:

            if not isinstance(person, dict):
                raise ValidationError("the person field must be a JSON Field")

            affiliations = person.get("affiliations", [])

            if isinstance(affiliations, list):
                for affiliation in affiliations:

                    if isinstance(affiliation, dict):
                        if affiliation != {}: # if not empty dict should have at least affiliation name
                            affiliation_name = affiliation.get("affiliation", None)

                            if affiliation_name is None:
                                raise ValidationError('affiliation needs at least a name: "affiliation": <name>')
                        else:
                            raise ValidationError("empty affiliations are not valid")
                    else:
                        raise ValidationError("each affiliation must be a JSON field")

            else:
                raise ValidationError("affiliations must be a list of JSON Fields")



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-04-10 16:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('daiquiri_jobs', '0006_owner_blank_true'),
    ]

    operations = [
        migrations.CreateModel(
            name='QueryJob',
            fields=[
                ('job_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='daiquiri_jobs.Job')),
                ('tablename', models.CharField(max_length=256)),
                ('query', models.TextField()),
                ('queue', models.CharField(max_length=16)),
                ('nrows', models.IntegerField(blank=True, null=True)),
                ('size', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'verbose_name_plural': 'QueryJobs',
                'permissions': (('view_queryjob', 'Can view QueryJob'),),
                'verbose_name': 'QueryJob',
            },
            bases=('daiquiri_jobs.job',),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _


class Datalink(models.Model):

    datalink_id = models.BigAutoField(primary_key=True)
    ID = models.CharField(
        max_length=256,
        db_index=True,
        verbose_name=_('Identifier'),
    )
    access_url = models.CharField(
        max_length=256,
        verbose_name=_('Access URL'),
    )
    service_def = models.CharField(
        max_length=80, blank=True,
        verbose_name=_('Service definition'),
    )
    error_message = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Error message'),
    )
    description = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Description'),
    )
    semantics = models.CharField(
        max_length=80, db_index=True, blank=True,
        verbose_name=_('Semantics'),
    )
    content_type = models.CharField(
        max_length=80, db_index=True,
        verbose_name=_('Content type'),
    )
    content_length = models.BigIntegerField(
        null=True,
        verbose_name=_('Content length'),
    )

    class Meta:
        db_table = 'datalink'

    def __str__(self):
        return self.ID



import pytest

from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'users': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 302
    }
}


@pytest.mark.parametrize(('username', 'password'), users)
def test_users(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('auth:users')
    response = client.get(url)
    assert response.status_code == status_map['users'][username]



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class QueryConfig(AppConfig):
    name = 'daiquiri.query'
    label = 'daiquiri_query'
    verbose_name = 'Query'

    def ready(self):
        if not settings.QUERY_DOWNLOAD_DIR:
            raise ImproperlyConfigured('QUERY_DOWNLOAD_DIR is not set')
        if not settings.QUERY_DOWNLOAD_DIR:
            raise ImproperlyConfigured('QUERY_UPLOAD_DIR is not set')

        from . import handlers  # noqa: F401



# Generated by Django 2.1.4 on 2019-05-28 09:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0021_django2'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queryjob',
            name='actual_query',
            field=models.TextField(blank=True, default=''),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='native_query',
            field=models.TextField(blank=True, default=''),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='query',
            field=models.TextField(blank=True, default=''),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='query_language',
            field=models.CharField(blank=True, default='', max_length=16),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='queue',
            field=models.CharField(blank=True, default='', max_length=16),
            preserve_default=False,
        ),
    ]



from django.utils.timezone import now

from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from daiquiri.core.utils import get_client_ip
from daiquiri.stats.models import Record

from .adapter import CutOutAdapter
from .permissions import HasPermission
from .renderers import CutOutErrorRenderer


class CutOutViewSet(APIView):

    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)
    renderer_classes = (CutOutErrorRenderer, JSONRenderer)

    def get(self, request, resource=None):

        adapter = CutOutAdapter()
        adapter.clean(request, resource)

        if request.GET.get('download', True):
            # create a stats record for this cutout
            Record.objects.create(
                time=now(),
                resource_type='CUTOUT',
                resource=adapter.args,
                client_ip=get_client_ip(request),
                user=request.user if request.user.is_authenticated else None
            )

            # stream the table
            return adapter.stream()
        else:
            # send an empty response
            return Response()



from django.urls import reverse

from daiquiri.core.utils import get_admin_emails, get_permission_emails, send_mail


def get_full_name(user):
    if user.first_name and user.last_name:
        return user.first_name + ' ' + user.last_name
    else:
        return user.username


def get_manager_emails():
    return get_permission_emails((
        'daiquiri_auth.view_profile',
    ))


def send_request_confirmation(request, user):
    '''
    Sends an email to the admins once a users email was validated.
    '''
    send_mail(request, 'account/email/request_confirmation', {
        'user': user,
        'users_url': request.build_absolute_uri(reverse('auth:users'))
    }, get_manager_emails())


def send_request_activation(request, user):
    '''
    Sends an email to the admins once a users email was validated.
    '''
    send_mail(request, 'account/email/request_activation', {
        'user': user,
        'users_url': request.build_absolute_uri(reverse('auth:users'))
    }, get_manager_emails())


def send_notify_confirmation(request, user):
    '''
    Sends an email to the admins once a user was confirmed.
    '''
    send_mail(request, 'account/email/notify_confirmation', {
        'user': user,
        'request_user': request.user
    }, get_manager_emails())


def send_notify_rejection(request, user):
    '''
    Sends an email to the admins once a user was rejected.
    '''
    send_mail(request, 'account/email/notify_rejection', {
        'user': user,
        'request_user': request.user
    }, get_manager_emails())


def send_notify_activation(request, user):
    '''
    Sends an email to the admins once a user was activated.
    '''
    send_mail(request, 'account/email/notify_activation', {
        'user': user,
        'request_user': request.user
    }, get_manager_emails())


def send_activation(request, user):
    '''
    Sends an email to the once he/she was activated.
    '''
    send_mail(request, 'account/email/activation', {
        'user': user,
        'login_url': request.build_absolute_uri(reverse('account_login'))
    }, [user.email])


def send_notify_password_changed(request, user):
    '''
    Sends an email to the admins once a user updated his/her account.
    '''
    send_mail(request, 'account/email/notify_password_changed', {
        'user': user
    }, get_admin_emails())



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-10-25 10:10
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0013_remove_directory'),
    ]

    operations = [
        migrations.RenameField(
            model_name='database',
            old_name='pid',
            new_name='doi',
        ),
        migrations.RenameField(
            model_name='table',
            old_name='pid',
            new_name='doi',
        ),
    ]



from django.conf import settings


def get_curation():
    return {
        'publisher': settings.SITE_PUBLISHER,
        'date': settings.SITE_UPDATED,
        'creator': {
            'name': settings.SITE_CREATOR,
            'logo': settings.SITE_LOGO_URL,
        },
        'contact': settings.SITE_CONTACT
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-01-09 14:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_files', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='directory',
            name='depth',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='directory',
            name='path',
            field=models.CharField(blank=True, help_text='Path of the directory.', max_length=256, verbose_name='Path'),
        ),
    ]



# Generated by Django 2.1.4 on 2019-06-12 10:34

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0022_blank_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='queryjob',
            name='uploads',
            field=jsonfield.fields.JSONField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-08-16 12:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Column',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('table_name', models.CharField(max_length=256)),
                ('column_name', models.CharField(max_length=256)),
                ('datatype', models.CharField(max_length=256)),
                ('arraysize', models.IntegerField(blank=True, null=True)),
                ('size', models.IntegerField(blank=True, null=True)),
                ('description', models.CharField(blank=True, max_length=256, null=True)),
                ('utype', models.CharField(blank=True, max_length=256, null=True)),
                ('unit', models.CharField(blank=True, max_length=256, null=True)),
                ('ucd', models.CharField(blank=True, max_length=256, null=True)),
                ('principal', models.BooleanField(default=False)),
                ('indexed', models.BooleanField(default=False)),
                ('std', models.BooleanField(default=False)),
                ('column_index', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'db_table': 'columns',
            },
        ),
        migrations.CreateModel(
            name='Schema',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('schema_name', models.CharField(max_length=256)),
                ('utype', models.CharField(blank=True, max_length=256, null=True)),
                ('description', models.CharField(blank=True, max_length=256, null=True)),
            ],
            options={
                'db_table': 'schemas',
            },
        ),
        migrations.CreateModel(
            name='Table',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('schema_name', models.CharField(max_length=256)),
                ('table_name', models.CharField(max_length=256)),
                ('table_type', models.CharField(max_length=256)),
                ('utype', models.CharField(blank=True, max_length=256, null=True)),
                ('description', models.CharField(blank=True, max_length=256, null=True)),
                ('table_index', models.IntegerField(blank=True, null=True)),
                ('schema', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tables', to='daiquiri_tap.Schema')),
            ],
            options={
                'db_table': 'tables',
            },
        ),
        migrations.AddField(
            model_name='column',
            name='table',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='columns', to='daiquiri_tap.Table'),
        ),
    ]



from django.conf import settings
from django.urls import reverse

from daiquiri.core.constants import ACCESS_LEVEL_PUBLIC
from daiquiri.core.vo import get_curation
from daiquiri.metadata.models import Schema
from daiquiri.query.utils import get_quota

from .serializers import SchemaSerializer


def get_resource():
    return {
        'service': 'tap',
        'identifier': f'ivo://{settings.SITE_IDENTIFIER}/tap',
        'title': f'{settings.SITE_TITLE} TAP Service',
        'curation': get_curation(),
        'content': {
            'subjects': settings.TAP_SUBJECTS,
            'type': 'Catalog',
            'description': f'The TAP Service registry for {settings.SITE_IDENTIFIER}.',
            'referenceURL': settings.SITE_URL.rstrip('/') + reverse('tap:root').rstrip('/')
        },
        'capabilities': get_capabilities(),
        'tableset': get_tableset(),
        'created': settings.SITE_CREATED,
        'updated': settings.SITE_UPDATED,
        'type': 'vs:CatalogResource',
        'status': 'active'
    }


def get_availability():
    return {
        'available': 'true',
        'note': 'service is accepting queries'
    }


def get_capabilities():
    return [
        {
            'id': 'ivo://ivoa.net/std/TAP',
            'interface': {
                'type': 'vs:ParamHTTP',
                'role': 'std',
                'access_url': {
                    'use': 'base',
                    'url': settings.SITE_URL.rstrip('/') + reverse('tap:root').rstrip('/')
                }
            },
            'languages': [{
                'name': language['key'],
                'version': language['version'],
                'description': language['description'],
            } for language in settings.QUERY_LANGUAGES],
            'upload_methods': [
                'ivo://ivoa.net/std/TAPRegExt#upload-inline',
                'ivo://ivoa.net/std/TAPRegExt#upload-https',
            ] if settings.TAP_UPLOAD else [],
            'upload_limit': int(get_quota(None, quota_settings='QUERY_UPLOAD_LIMIT'))
        },
        {
            'id': 'ivo://ivoa.net/std/TAP#async-1.1',
            'interface': {
                'type': 'vs:ParamHTTP',
                'role': 'std',
                'version': '1.1',
                'access_url': {
                    'use': 'base',
                    'url': settings.SITE_URL.rstrip('/') + reverse('tap:async-list').rstrip('/')
                }
            }
        },
        {
            'id': 'ivo://ivoa.net/std/TAP#sync-1.1',
            'interface': {
                'type': 'vs:ParamHTTP',
                'role': 'std',
                'version': '1.1',
                'access_url': {
                    'use': 'base',
                    'url': settings.SITE_URL.rstrip('/') + reverse('tap:sync-list').rstrip('/')
                }
            }
        },
        {
            'id': 'ivo://ivoa.net/std/VOSI#capabilities',
            'interface': {
                'type': 'vs:ParamHTTP',
                'access_url': {
                    'use': 'full',
                    'url': settings.SITE_URL.rstrip('/') + reverse('tap:capabilities').rstrip('/')
                }
            }
        },
        {
            'id': 'ivo://ivoa.net/std/VOSI#tables',
            'interface': {
                'type': 'vs:ParamHTTP',
                'access_url': {
                    'use': 'full',
                    'url': settings.SITE_URL.rstrip('/') + reverse('tap:tables').rstrip('/')
                }
            }
        },
        {
            'id': 'ivo://ivoa.net/std/DALI#examples',
            'interface': {
                'type': 'vr:WebBrowser',
                'access_url': {
                    'use': 'full',
                    'url': settings.SITE_URL.rstrip('/') + reverse('tap:examples').rstrip('/')
                }
            }
        }
    ]


def get_tableset():
    queryset = Schema.objects.filter(metadata_access_level=ACCESS_LEVEL_PUBLIC, published__isnull=False)
    serializer = SchemaSerializer(queryset, many=True)
    return serializer.data



from __future__ import unicode_literals

from django.db import migrations


def migrate_jobs(apps, schema_editor):
    QueryJob = apps.get_model('daiquiri_query', 'QueryJob')
    Record = apps.get_model('daiquiri_stats', 'Record')

    for job in QueryJob.objects.all():
        Record.objects.create(
            time=job.end_time,
            resource_type='QUERY_JOB',
            resource={
                'job_id': job.id,
                'job_type': job.job_type,
                'tables': job.metadata['source_tables'] if 'source_tables' in job.metadata else []
            },
            client_ip=job.client_ip,
            user=job.owner
        )

class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_stats', '0001_initial'),
        ('daiquiri_jobs', '0012_meta'),
        ('daiquiri_query', '0013_refactoring')
    ]

    operations = [
        migrations.RunPython(migrate_jobs)
    ]



from django.urls import path, re_path
from django.views.generic import TemplateView

from .views import availability, capabilities, resource
from .viewsets import ConeSearchView

app_name = 'conesearch'

urlpatterns = [
    path('', TemplateView.as_view(template_name='conesearch/root.html'), name='root'),
    path('resource', resource, name='resource'),
    path('availability', availability, name='availability'),
    path('capabilities', capabilities, name='capabilities'),
    re_path(r'^api/(?P<resource>.+)/$', ConeSearchView.as_view(), name='search'),
]



import pytest

from django.test import override_settings
from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'query': {
        'admin': 200, 'manager': 200, 'user': 200, 'anonymous': 302
    },
    'examples': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 302
    }
}


@pytest.mark.parametrize(('username', 'password'), users)
def test_query(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('query:query')
    response = client.get(url)
    assert response.status_code == status_map['query'][username]


@override_settings(QUERY_ANONYMOUS=True)
def test_list_anonymous(db, client):
    url = reverse('query:query')
    response = client.get(url)
    assert response.status_code == 200



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-01 13:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0006_owner_blank_true'),
    ]

    operations = [
        migrations.AddField(
            model_name='job',
            name='creation_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



from django.conf import settings
from django.db.models import Max

from rest_framework import filters, status, viewsets
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from django_filters.rest_framework import DjangoFilterBackend

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.constants import ACCESS_LEVEL_CHOICES
from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.utils import get_model_field_meta
from daiquiri.core.viewsets import ChoicesViewSet

from .models import Column, Function, Schema, Table
from .serializers import ColumnSerializer, FunctionSerializer, SchemaSerializer, TableSerializer
from .serializers.export import FunctionSerializer as ExportFunctionSerializer
from .serializers.export import SchemaSerializer as ExportSchemaSerializer
from .serializers.management import FunctionSerializer as ManagementFunctionSerializer
from .serializers.management import SchemaSerializer as ManagementSchemaSerializer
from .serializers.user import FunctionSerializer as UserFunctionSerializer
from .serializers.user import SchemaSerializer as UserSchemaSerializer


class SchemaViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Schema.objects.all()
    serializer_class = SchemaSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        schema = serializer.save()
        schema.order = (Schema.objects.aggregate(order=Max('order'))['order'] or 0) + 1
        schema.save()

        if request.data.get('discover'):
            adapter = DatabaseAdapter()

            for table_order, table_metadata in enumerate(adapter.fetch_tables(schema.name)):
                table_metadata['order'] = table_order
                table_metadata['schema'] = schema.id
                table_metadata['groups'] = [group.id for group in schema.groups.all()]
                for key in ['license', 'access_level', 'metadata_access_level']:
                    table_metadata[key] = getattr(schema, key)

                table_serializer = TableSerializer(data=table_metadata)
                if table_serializer.is_valid():
                    table = table_serializer.save()
                    table.discover(adapter)
                    table.save()

                    for column_order, column_metadata in enumerate(adapter.fetch_columns(schema.name, table.name)):
                        column_metadata['order'] = column_order
                        column_metadata['table'] = table.id
                        column_metadata['groups'] = [group.id for group in table.groups.all()]
                        for key in ['access_level', 'metadata_access_level']:
                            column_metadata[key] = getattr(table, key)

                        column_serializer = ColumnSerializer(data=column_metadata)
                        if column_serializer.is_valid():
                            column = column_serializer.save()
                            column.discover(adapter)
                            column.save()

        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=False)
    def management(self, request):
        queryset = Schema.objects.all()
        serializer = ManagementSchemaSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'], permission_classes=[])
    def user(self, request):
        # filter the schemas which are published for the groups of the user
        queryset = Schema.objects.filter_by_access_level(self.request.user)
        serializer = UserSchemaSerializer(queryset, context={'request': request}, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'], url_path='export', url_name='export-detail')
    def export_list(self, request):
        queryset = Schema.objects.all()
        serializer = ExportSchemaSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=True, methods=['get'], url_path='export', url_name='export-detail')
    def export_detail(self, request, pk=None):
        queryset = Schema.objects.get(pk=pk)
        serializer = ExportSchemaSerializer(queryset)
        return Response(serializer.data)


class TableViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Table.objects.all()
    serializer_class = TableSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        table = serializer.save()
        table.order = (table.schema.tables.aggregate(order=Max('order'))['order'] or 0) + 1
        table.save()

        if request.data.get('discover'):
            adapter = DatabaseAdapter()
            table.discover(adapter)
            table.save()

            for column_order, column_metadata in enumerate(adapter.fetch_columns(table.schema.name, table.name)):
                column_metadata['order'] = column_order
                column_metadata['table'] = table.id
                column_metadata['groups'] = [group.id for group in table.groups.all()]
                for key in ['access_level', 'metadata_access_level']:
                    column_metadata[key] = getattr(table, key)

                column_serializer = ColumnSerializer(data=column_metadata)
                if column_serializer.is_valid():
                    column = column_serializer.save()
                    column.discover(adapter)
                    column.save()

        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=True, methods=['post'])
    def discover(self, request, pk=None):
        adapter = DatabaseAdapter()
        table = self.get_object()
        table.discover(adapter)
        table.save()
        return Response(self.get_serializer(table).data)


class ColumnViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Column.objects.all()
    serializer_class = ColumnSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        column = serializer.save()
        column.order = (column.table.columns.aggregate(order=Max('order'))['order'] or 0) + 1
        column.save()

        if request.data.get('discover'):
            adapter = DatabaseAdapter()
            column.discover(adapter)
            column.save()

        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=True, methods=['post'])
    def discover(self, request, pk=None):
        adapter = DatabaseAdapter()
        column = self.get_object()
        column.discover(adapter)
        column.save()
        return Response(self.get_serializer(column).data)


class FunctionViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Function.objects.all()
    serializer_class = FunctionSerializer

    filter_backends = (DjangoFilterBackend, filters.SearchFilter, filters.OrderingFilter)
    filterset_fields = ('name', 'access_level', 'metadata_access_level')
    search_fields = ('name', 'description')
    ordering_fields = ('name', 'access_level', 'metadata_access_level')

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        function = serializer.save()
        function.order = (Function.objects.aggregate(order=Max('order'))['order'] or 0) + 1
        function.save()

        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=False, methods=['get'])
    def management(self, request):
        queryset = Function.objects.all()
        serializer = ManagementFunctionSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'])
    def export(self, request):
        queryset = Function.objects.all()
        serializer = ExportFunctionSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'], permission_classes=[])
    def user(self, request):
        queryset = Function.objects.filter_by_access_level(self.request.user)
        serializer = UserFunctionSerializer(queryset, many=True)
        return Response(serializer.data)


class LicenseViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = settings.LICENSE_CHOICES


class AccessLevelViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = ACCESS_LEVEL_CHOICES


class MetaViewSet(viewsets.ViewSet):
    permission_classes = (IsAuthenticated, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    def list(self, request, *args, **kwargs):
        return Response(get_model_field_meta(Schema, Table, Column, Function))



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-03 15:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0004_function_query_string'),
    ]

    operations = [
        migrations.AddField(
            model_name='database',
            name='attribution',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='database',
            name='license',
            field=models.CharField(blank=True, choices=[(b'CC0', b'CC0 1.0 Universal (CC0 1.0)'), (b'PD', b'Public Domain Mark'), (b'BY', b'Attribution 4.0 International (CC BY 4.0)'), (b'BY_SA', b'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), (b'BY_ND', b'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), (b'BY_NC', b'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), (b'BY_NC_SA', b'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), (b'BY_NC_ND', b'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True),
        ),
        migrations.AddField(
            model_name='database',
            name='long_description',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='database',
            name='pid',
            field=models.URLField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name='database',
            name='title',
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name='table',
            name='attribution',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='table',
            name='license',
            field=models.CharField(blank=True, choices=[(b'CC0', b'CC0 1.0 Universal (CC0 1.0)'), (b'PD', b'Public Domain Mark'), (b'BY', b'Attribution 4.0 International (CC BY 4.0)'), (b'BY_SA', b'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), (b'BY_ND', b'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), (b'BY_NC', b'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), (b'BY_NC_SA', b'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), (b'BY_NC_ND', b'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True),
        ),
        migrations.AddField(
            model_name='table',
            name='long_description',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='table',
            name='pid',
            field=models.URLField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name='table',
            name='title',
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
    ]



# ruff: noqa: F403, F405, I001, RUF005
'''
Generic settings to be used with daiquiri-admin outside of an daiquiri-app.
'''
from pathlib import Path

from daiquiri.core.settings.django import *
from daiquiri.core.settings.daiquiri import *
from daiquiri.core.settings.logging import *

from daiquiri.auth.settings import *
from daiquiri.conesearch.settings import *
from daiquiri.contact.settings import *
from daiquiri.cutout.settings import *
from daiquiri.datalink.settings import *
from daiquiri.files.settings import *
from daiquiri.metadata.settings import *
from daiquiri.oai.settings import *
from daiquiri.query.settings import *
from daiquiri.registry.settings import *
from daiquiri.serve.settings import *
from daiquiri.stats.settings import *
from daiquiri.tap.settings import *

INSTALLED_APPS = DJANGO_APPS + [
    'daiquiri.auth',
    'daiquiri.conesearch',
    'daiquiri.contact',
    'daiquiri.core',
    'daiquiri.datalink',
    'daiquiri.files',
    'daiquiri.jobs',
    'daiquiri.metadata',
    'daiquiri.oai',
    'daiquiri.query',
    'daiquiri.registry',
    'daiquiri.serve',
    'daiquiri.stats',
    'daiquiri.tap',
    'daiquiri.uws',
]

ROOT_URLCONF = ''

DATABASES = {}

STATIC_ROOT = 'static_root'

FILES_BASE_PATH = QUERY_DOWNLOAD_DIR = Path().cwd()



# Generated by Django 3.2.12 on 2022-02-04 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0025_alter_example_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queryarchivejob',
            name='files',
            field=models.JSONField(help_text='List of files in the archive.', verbose_name='Files'),
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='metadata',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='uploads',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.11 on 2022-01-17 14:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_datalink', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datalink',
            name='content_length',
            field=models.IntegerField(null=True, verbose_name='Content length'),
        ),
    ]



from django.urls import include, path, re_path
from django.views.generic import TemplateView

from daiquiri.jobs.routers import JobRouter
from daiquiri.query.viewsets import AsyncQueryJobViewSet, SyncQueryJobViewSet

from .views import availability, capabilities, examples, resource, tables

app_name = 'tap'

router = JobRouter(trailing_slash=False)
router.register(r'sync', SyncQueryJobViewSet, basename='sync')
router.register(r'async', AsyncQueryJobViewSet, basename='async')

urlpatterns = [
    path('', TemplateView.as_view(template_name='tap/root.html'), name='root'),
    path('resource', resource, name='resource'),
    path('availability', availability, name='availability'),
    path('capabilities', capabilities, name='capabilities'),
    path('tables', tables, name='tables'),
    path('examples', examples, name='examples'),

    re_path(r'^', include(router.urls)),
]



# Generated by Django 2.1.3 on 2018-11-05 16:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0015_job_index'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='job',
            options={'ordering': ('start_time',), 'verbose_name': 'Job', 'verbose_name_plural': 'Jobs'},
        ),
    ]



from django.urls import include, path, re_path
from django.views.generic import TemplateView

from daiquiri.jobs.routers import JobRouter

from .views import availability, capabilities, datalink, datalink_semantics
from .viewsets import SyncDatalinkJobViewSet

app_name = 'datalink'

router = JobRouter(trailing_slash=False)
router.register(r'links', SyncDatalinkJobViewSet, basename='link')

urlpatterns = [
    path('', TemplateView.as_view(template_name='datalink/root.html'), name='root'),
    path('semantics', datalink_semantics, name='datalink-semantics'),
    path('<path:ID>/', datalink, name='datalink'),
    path('availability', availability, name='availability'),
    path('capabilities', capabilities, name='capabilities'),

    re_path(r'^', include(router.urls)),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-11 12:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0007_queryjob_metadata'),
    ]

    operations = [
        migrations.AddField(
            model_name='queryjob',
            name='pid',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-13 09:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0007_creation_time'),
    ]

    operations = [
        migrations.AddField(
            model_name='job',
            name='error_summary',
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
    ]



from django.conf import settings

from daiquiri.core.utils import import_class


def get_resolver():
    if settings.SERVE_RESOLVER:
        return import_class(settings.SERVE_RESOLVER)()
    else:
        return None



from celery import Task as CeleryTask


class Task(CeleryTask):

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        import logging
        logger = logging.getLogger('daiquiri')
        logger.error(einfo)



from django.conf import settings
from django.contrib.auth.models import Group
from django.shortcuts import get_object_or_404

from rest_framework import filters, mixins, viewsets
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.decorators import action
from rest_framework.exceptions import MethodNotAllowed
from rest_framework.response import Response

from django_filters.rest_framework import DjangoFilterBackend

from daiquiri.core.paginations import ListPagination
from daiquiri.core.permissions import HasModelPermission

from .models import Profile
from .permissions import GroupViewPermission, IsAuthManager
from .serializers import GroupSerializer, ProfileSerializer


class ProfileViewSet(mixins.UpdateModelMixin, mixins.ListModelMixin,
                     mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Profile.objects.all()
    serializer_class = ProfileSerializer
    pagination_class = ListPagination

    filter_backends = (filters.SearchFilter, filters.OrderingFilter, DjangoFilterBackend)
    ordering_fields = (
        'user__username',
        'user__email',
        'user__first_name',
        'user__last_name',
        'user__date_joined'
    )
    search_fields = (
        'user__username',
        'user__email',
        'user__first_name',
        'user__last_name'
    )
    filterset_fields = (
        'is_pending',
        'is_confirmed',
        'user__is_active',
        'user__is_staff',
        'user__is_superuser',
    )

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def confirm(self, request, pk=None):
        if not settings.AUTH_WORKFLOW:
            raise MethodNotAllowed('put')

        profile = get_object_or_404(Profile, pk=pk)
        profile.confirm(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def reject(self, request, pk=None):
        if not settings.AUTH_WORKFLOW:
            raise MethodNotAllowed('put')

        profile = get_object_or_404(Profile, pk=pk)
        profile.reject(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def activate(self, request, pk=None):
        if not settings.AUTH_WORKFLOW:
            raise MethodNotAllowed('put')

        profile = get_object_or_404(Profile, pk=pk)
        profile.activate(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def disable(self, request, pk=None):
        profile = get_object_or_404(Profile, pk=pk)
        profile.disable(request)
        return Response(self.get_serializer(profile).data)

    @action(detail=True, methods=['put'], permission_classes=[HasModelPermission])
    def enable(self, request, pk=None):
        profile = get_object_or_404(Profile, pk=pk)
        profile.enable(request)
        return Response(self.get_serializer(profile).data)


class SettingsViewSet(viewsets.GenericViewSet):
    permission_classes = (IsAuthManager, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = {
        'AUTH_WORKFLOW': settings.AUTH_WORKFLOW,
        'AUTH_DETAIL_KEYS': settings.AUTH_DETAIL_KEYS
    }

    def list(self, request, *args, **kwargs):
        return Response(self.get_queryset())


class GroupViewSet(viewsets.ReadOnlyModelViewSet):
    permission_classes = (GroupViewPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = Group.objects.order_by('name')
    serializer_class = GroupSerializer



from django.template.loader import render_to_string

from rest_framework import serializers

from .models import ContactMessage


class ContactMessageSerializer(serializers.ModelSerializer):

    mailto = serializers.SerializerMethodField()

    class Meta:
        model = ContactMessage
        fields = (
            'id',
            'author',
            'email',
            'subject',
            'status',
            'status_label',
            'created',
            'created_label',
            'message',
            'user',
            'mailto'
        )

    def get_mailto(self, obj):
        return render_to_string('contact/messages_mailto.html', {
            'request': self.context.get('request'),
            'message': obj
        })



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-01 13:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0003_query_language_and_actual_query'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='queryjob',
            options={'ordering': ('start_time',), 'permissions': (('view_queryjob', 'Can view QueryJob'),), 'verbose_name': 'QueryJob', 'verbose_name_plural': 'QueryJobs'},
        ),
        migrations.RenameField(
            model_name='queryjob',
            old_name='tablename',
            new_name='table_name',
        ),
        migrations.AddField(
            model_name='queryjob',
            name='database_name',
            field=models.CharField(default='daiquiri_user', max_length=256),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='queryjob',
            name='query_language',
            field=models.CharField(choices=[('mysql', 'MySQL SQL'), ('adql', 'ADQL')], max_length=8),
        ),
    ]



from django.conf import settings
from django.urls import reverse

from rest_framework import serializers

from daiquiri.core.serializers import JSONListField
from daiquiri.metadata.models import Schema, Table


class DataciteSerializer(serializers.ModelSerializer):

    identifier = serializers.SerializerMethodField()
    creators = JSONListField(default=[])
    title = serializers.SerializerMethodField()
    publisher = serializers.ReadOnlyField(default=settings.SITE_PUBLISHER)
    publication_year = serializers.SerializerMethodField()
    subjects = serializers.ReadOnlyField(default=settings.SITE_SUBJECTS)
    contributors = JSONListField(default=[])
    language = serializers.ReadOnlyField(default=settings.SITE_LANGUAGE)
    alternate_identifiers = serializers.SerializerMethodField()
    related_identifiers = JSONListField(default=[])
    resource_type = serializers.SerializerMethodField()
    formats = serializers.SerializerMethodField()
    sizes = serializers.SerializerMethodField()
    license_url = serializers.SerializerMethodField()
    license_label = serializers.SerializerMethodField()
    license_identifier = serializers.SerializerMethodField()
    description = serializers.SerializerMethodField()

    def get_publication_year(self, obj):
        return obj.published.year if obj.published else None

    def get_alternate_identifiers(self, obj):
        raise NotImplementedError()

    def get_formats(self, obj):
        if hasattr(settings, 'QUERY_DOWNLOAD_FORMATS'):
            return [f['content_type'] for f in settings.QUERY_DOWNLOAD_FORMATS]
        else:
            return []

    def get_sizes(self, obj):
        raise NotImplementedError()

    def get_license_url(self, obj):
        return settings.LICENSE_URLS.get(obj.license)

    def get_license_label(self, obj):
        return str(dict(settings.LICENSE_CHOICES).get(obj.license))

    def get_license_identifier(self, obj):
        return settings.LICENSE_IDENTIFIERS.get(obj.license)

    def get_description(self, obj):
        return obj.long_description or obj.description


class DataciteSchemaSerializer(DataciteSerializer):

    class Meta:
        model = Schema
        fields = (
            'identifier',
            'creators',
            'title',
            'publisher',
            'publication_year',
            'subjects',
            'contributors',
            'published',
            'updated',
            'language',
            'resource_type',
            'alternate_identifiers',
            'related_identifiers',
            'sizes',
            'formats',
            'license',
            'license_url',
            'license_label',
            'license_identifier',
            'description',
        )

    def get_title(self, obj):
        return obj.title or obj.name

    def get_identifier(self, obj):
        return obj.doi or 'schemas/%i' % obj.pk

    def get_alternate_identifiers(self, obj):
        url = self.context['request'].build_absolute_uri(reverse('metadata:schema', args=[obj.name]))
        return [{
            'alternate_identifier': url,
            'alternate_identifier_type': 'DOI Landing Page'
        }]

    def get_resource_type(self, obj):
        return 'Database schema'

    def get_sizes(self, obj):
        tables = obj.tables.filter_by_metadata_access_level(self.context['request'].user)
        return ['%i tables' % tables.count()]


class DataciteTableSerializer(DataciteSerializer):

    class Meta:
        model = Table
        fields = '__all__'
        fields = (
            'identifier',
            'creators',
            'title',
            'publisher',
            'publication_year',
            'subjects',
            'contributors',
            'published',
            'updated',
            'language',
            'resource_type',
            'alternate_identifiers',
            'related_identifiers',
            'sizes',
            'formats',
            'license',
            'license_url',
            'license_label',
            'license_identifier',
            'description',
        )

    def get_title(self, obj):
        return obj.title or f'{obj.schema.name}.{obj.name}'

    def get_identifier(self, obj):
        return obj.doi or 'tables/%i' % obj.pk

    def get_resource_type(self, obj):
        return 'Database table'

    def get_alternate_identifiers(self, obj):
        url = self.context['request'].build_absolute_uri(reverse('metadata:table', args=[obj.schema.name, obj.name]))
        return [{
            'alternate_identifier': url,
            'alternate_identifier_type': 'DOI Landing Page'
        }]

    def get_sizes(self, obj):
        # filter the columns which are published for the groups of the user
        if not settings.METADATA_COLUMN_PERMISSIONS:
            columns = obj.columns.all()
        else:
            columns = obj.columns.filter_by_access_level(self.context['request'].user)

        return ['%i columns' % columns.count(), '%i rows' % (obj.nrows or 0)]



from django.core.management.base import BaseCommand
from django.db.utils import ProgrammingError

from rest_framework.exceptions import ValidationError

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.query.models import QueryJob


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--user', help='Only fix jobs for this user.')
        parser.add_argument('--queue', help='Name of the queue to use.')
        parser.add_argument('--dry', action='store_true', help='Perform a dryrun.')

    def handle(self, *args, **options):
        # look for completed jobs with no table
        queryset = QueryJob.objects.filter(phase=QueryJob.PHASE_COMPLETED)

        if options['user']:
            queryset = queryset.filter(owner__username=options['user'])

        for job in queryset:
            try:
                DatabaseAdapter().fetch_size(job.schema_name, job.table_name)
            except ProgrammingError:
                try:
                    job.phase = QueryJob.PHASE_PENDING

                    if options['queue']:
                        job.queue = options['queue']

                    job.process()

                    print(f'Run {job.id} by {job.owner} again.')

                    if not options['dry']:
                        job.run()

                except ValidationError as e:
                    job.phase = QueryJob.PHASE_ERROR

                    job.error_summary = ''
                    for key, errors in e.detail.items():
                        try:
                            job.error_summary += ''.join(errors['messages'])
                        except TypeError:
                            job.error_summary += ''.join(errors)

                    print(f'Error for {job.id} by {job.owner}: {job.error_summary}')

                    if not options['dry']:
                        job.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-05-26 12:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_auth', '0006_profile_ordering'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='is_confirmed',
            field=models.BooleanField(default=True),
        ),
    ]



import daiquiri.core.env as env

CONESEARCH_ADAPTER = 'daiquiri.conesearch.adapter.SimpleConeSearchAdapter'
CONESEARCH_ANONYMOUS = False

CONESEARCH_SCHEMA = env.get('CONESEARCH_SCHEMA')
CONESEARCH_TABLE = env.get('CONESEARCH_TABLE')

CONESEARCH_SUBJECTS = ['cone search']



import daiquiri.core.env as env

QUERY_DOWNLOAD_DIR = env.get_abspath('QUERY_DOWNLOAD_DIR')
QUERY_UPLOAD_DIR = env.get_abspath('QUERY_UPLOAD_DIR')

QUERY_ANONYMOUS = False
QUERY_USER_SCHEMA_PREFIX = 'daiquiri_user_'
QUERY_QUOTA = {
    'anonymous': '100Mb',
    'user': '10000Mb',
    'users': {},
    'groups': {}
}
QUERY_SYNC_TIMEOUT = 5
QUERY_MAX_ACTIVE_JOBS = {
    'anonymous': '1'
}
QUERY_QUEUES = [
    {
        'key': 'short',
        'label': '30 seconds',
        'timeout': 30,
        'access_level': 'PUBLIC',
        'groups': []
    },
    {
        'key': 'long',
        'label': '1 Hour',
        'timeout': 3600,
        'access_level': 'PUBLIC',
        'groups': []
    }
]
QUERY_LANGUAGES = [
    {
        'key': 'adql',
        'version': 2.0,
        'label': 'ADQL',
        'description': '',
        'quote_char': '"'
    }
]
QUERY_FORMS = [
    {
        'key': 'sql',
        'label': 'SQL query',
        'template': 'query/query_form_sql.html'
    },
    {
        'key': 'upload',
        'label': 'Upload VOTable',
        'template': 'query/query_form_upload.html'
    }
]
QUERY_PLOTS = [
    {
        'key': 'scatter_plot',
        'label': 'Scatter',
        'is_active': True,
    },
    {
        'key': 'scatter_cmap_plot',
        'label': 'Scatter (color coded)',
        'is_active': True,
    },
    {
        'key': 'histogram',
        'label': 'Histogram',
        'is_active': True,
    }
]

QUERY_DROPDOWNS = [
    {
        'key': 'schemas',
        'label': 'Database',
    },
    {
        'key': 'columns',
        'label': 'Columns',
    },
    {
        'key': 'functions',
        'label': 'Functions',
    },
    {
        'key': 'simbad',
        'label': 'Simbad',
        'options': {
            'url': 'http://simbad.u-strasbg.fr/simbad/sim-id'
        }
    },
    {
        'key': 'vizier',
        'label': 'VizieR',
        'options': {
            'url': 'http://vizier.u-strasbg.fr/viz-bin/votable',
            'catalogs': ['I/322A', 'I/259']
        }
    },
    {
        'key': 'examples',
        'label': 'Examples',
        'classes': 'ms-auto'
    }
]
QUERY_DOWNLOADS = [
    {
        'key': 'table',
        'model': 'daiquiri.query.models.DownloadJob',
        'params': ['format_key']
    },
    {
        'key': 'archive',
        'model': 'daiquiri.query.models.QueryArchiveJob',
        'params': ['column_name']
    }
]
QUERY_DEFAULT_DOWNLOAD_FORMAT = 'votable'
QUERY_DOWNLOAD_FORMATS = [
    {
        'key': 'votable',
        'extension': 'xml',
        'content_type': 'application/xml',
        'label': 'IVOA VOTable',
        'help': 'A XML file using the IVOA VOTable format. Use this option if you intend ' \
                'to use VO compatible software to further process the data.'
    },
    {
        'key': 'csv',
        'extension': 'csv',
        'content_type': 'text/csv',
        'label': 'Comma-separated Values',
        'help': 'A text file with a line for each row of the table. The fields are delimited ' \
                'by a comma and quoted by double quotes. Use this option for a later import ' \
                'into a spreadsheed application or a custom script. Use this option if you ' \
                'are unsure what to use.'
    },
    {
        'key': 'fits',
        'extension': 'fits',
        'content_type': 'application/fits',
        'label': 'FITS',
        'help': 'Flexible Image Transport System (FITS) file format.'
    }
]
QUERY_UPLOAD = True
QUERY_UPLOAD_LIMIT = {
    'anonymous': '10Mb',
    'user': '100Mb',
    'users': {},
    'groups': {}
}
QUERY_PROCESSOR_CACHE = True



from django.conf import settings
from django.views.generic import TemplateView

from daiquiri.core.views import ModelPermissionMixin

from .models import Record


class ManagementView(ModelPermissionMixin, TemplateView):
    template_name = 'stats/management.html'
    permission_required = 'daiquiri_stats.view_record'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context['resource_types'] = settings.STATS_RESOURCE_TYPES

        for resource_type in context['resource_types']:
            queryset = Record.objects.filter(resource_type=resource_type['key'])

            resource_type['count'] = queryset.count()
            resource_type['client_ips'] = queryset.order_by('client_ip').values('client_ip').distinct().count()
            resource_type['users'] = queryset.order_by('user').values('user').distinct().count()
            resource_type['size'] = sum(queryset.filter(size__gte=0).values_list('size', flat=True))

        return context



# Generated by Django 3.2.10 on 2021-12-10 11:24

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Datalink',
            fields=[
                ('datalink_id', models.BigAutoField(primary_key=True, serialize=False)),
                ('ID', models.CharField(db_index=True, max_length=256, verbose_name='Identifier')),
                ('access_url', models.CharField(max_length=256, verbose_name='Access URL')),
                ('service_def', models.CharField(blank=True, max_length=80, verbose_name='Service definition')),
                ('error_message', models.CharField(blank=True, max_length=256, verbose_name='Error message')),
                ('description', models.CharField(blank=True, max_length=256, verbose_name='Description')),
                ('semantics', models.CharField(blank=True, db_index=True, max_length=80, verbose_name='Semantics')),
                ('content_type', models.CharField(db_index=True, max_length=80, verbose_name='Content type')),
                ('content_length', models.IntegerField(verbose_name='Content length')),
            ],
            options={
                'db_table': 'datalink',
            },
        ),
    ]



# Generated by Django 4.2.5 on 2023-09-20 07:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_contact', '0006_alter_contactmessage_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='AnnouncementMessage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=100, null=True, verbose_name='Title')),
                ('announcement', models.TextField(verbose_name='Announcement')),
                ('announcement_type', models.CharField(choices=[('info', 'info'), ('warning', 'warning'), ('danger', 'urgent')], default='info', max_length=8, verbose_name='Announcement type')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('visible', models.BooleanField(default=False, verbose_name='Visible')),
                ('visibility_filter', models.CharField(default='no_filter', max_length=50, verbose_name='Visibility filter')),
            ],
            options={
                'verbose_name': 'Announcement message',
                'verbose_name_plural': 'Announcement messages',
                'ordering': ('-updated', 'title'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-04-04 14:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0013_job_error_summary'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='run_id',
            field=models.CharField(blank=True, default='', max_length=64),
        ),
    ]



from django.conf import settings
from django.http import FileResponse
from django.utils.translation import gettext_lazy as _

from rest_framework.exceptions import NotFound, ValidationError

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.adapter.stream import BaseServiceAdapter
from daiquiri.core.generators import generate_votable
from daiquiri.core.utils import import_class, make_query_dict_upper_case


def ConeSearchAdapter():
    return import_class(settings.CONESEARCH_ADAPTER)()


class BaseConeSearchAdapter(BaseServiceAdapter):

    keys = ['RA', 'DEC', 'SR']

    ranges = {
        'RA': {
            'min': 0,
            'max': 360
        },
        'DEC': {
            'min': -90,
            'max': 90
        },
        'SR': {
            'min': 0,
            'max': 10
        }
    }

    defaults = {
        'RA': 20,
        'DEC': 20,
        'SR': 10,
    }

    columns = [
        {
            'name': 'id',
            'ucd': 'meta.id; meta.main',
            'datatype': 'char',
            'arraysize': '*'
        },
        {
            'name': 'RA',
            'ucd': 'pos.eq.ra; meta.main',
            'datatype': 'double'
        },
        {
            'name': 'DEC',
            'ucd': 'pos.eq.dec; meta.main',
            'datatype': 'double'
        }
    ]

    max_records = 10000

    def get_resources(self):
        raise NotImplementedError()

    def clean(self, request, resource):
        raise NotImplementedError()

    def clean_args(self, data, errors):
        # parse RA, DEC, and SR arguments
        self.args = {}
        for key in ['RA', 'DEC', 'SR']:
            try:
                value = float(data[key])

                if self.ranges[key]['min'] <= value <= self.ranges[key]['max']:
                    self.args[key] = value
                else:
                    errors[key] = [_('This value must be between {min:g} and {max:g}.').format(**self.ranges[key])]

            except KeyError:
                errors[key] = [_('This field may not be blank.')]

            except ValueError:
                errors[key] = [_('This field must be a float.')]

    def stream(self):
        return FileResponse(
            generate_votable(DatabaseAdapter().fetchall(self.sql, self.args), self.columns),
            content_type='application/xml'
        )


class SimpleConeSearchAdapter(BaseConeSearchAdapter):

    def get_resources(self):
        return [settings.CONESEARCH_TABLE]

    def clean(self, request, resource):
        if resource not in self.get_resources():
            raise NotFound()

        adapter = DatabaseAdapter()

        self.sql = '''
SELECT {id_field}, {ra_field}, {dec_field}
FROM {schema}.{table}
WHERE
    {ra_field} BETWEEN (%(RA)s - 0.5 * %(SR)s) AND (%(RA)s + 0.5 * %(SR)s)
AND
    {dec_field} BETWEEN (%(DEC)s - 0.5 * %(SR)s) AND (%(DEC)s + 0.5 * %(SR)s)
LIMIT {limit}
'''.format(
            id_field=adapter.escape_identifier('id'),
            ra_field=adapter.escape_identifier('ra'),
            dec_field=adapter.escape_identifier('dec'),
            schema=settings.CONESEARCH_SCHEMA,
            table=settings.CONESEARCH_TABLE,
            limit=self.max_records,
        )

        self.args, errors = self.parse_query_dict(request)

        if errors:
            raise ValidationError(errors)


class TableConeSearchAdapter(BaseConeSearchAdapter):

    def clean(self, request, resource):
        from daiquiri.metadata.models import Schema, Table

        resources = self.get_resources()

        if resource not in resources:
            raise NotFound

        schema_name = resources[resource]['schema_name']
        table_name = resources[resource]['table_name']

        data = make_query_dict_upper_case(request.GET)
        errors = {}

        # check if the user is allowed to access the schema
        try:
            schema = Schema.objects.filter_by_access_level(request.user).get(name=schema_name)
        except Schema.DoesNotExist as e:
            raise NotFound from e

        # check if the user is allowed to access the table
        try:
            table = Table.objects.filter_by_access_level(request.user).filter(schema=schema).get(name=table_name)
        except Table.DoesNotExist as e:
            raise NotFound from e

        # fetch the columns according to the verbosity
        verb = data.get('VERB', '2')

        if verb == '1':
            self.columns = table.columns.filter(name__in=resources[resource]['column_names']).values()
        elif verb == '2':
            self.columns = table.columns.filter(principal=True).values()
        elif verb == '3':
            self.columns = table.columns.values()
        else:
            errors['VERB'] = [_('This field must be 1, 2, or 3.')]

        # construct sql query
        adapter = DatabaseAdapter()
        escaped_column_names = [adapter.escape_identifier(column['name']) for column in self.columns]
        self.sql = self.sql_pattern % {
            'schema': adapter.escape_identifier(schema_name),
            'table': adapter.escape_identifier(table_name),
            'columns': ', '.join(escaped_column_names)
        }

        # parse RA, DEC, and SR arguments
        self.args = {}
        for key in ['RA', 'DEC', 'SR']:
            try:
                value = float(data[key])

                if self.ranges[key]['min'] <= value <= self.ranges[key]['max']:
                    self.args[key] = value
                else:
                    errors[key] = [_('This value must be between {min:g} and {max:g}.').format(**self.ranges[key])]

            except KeyError:
                errors[key] = [_('This field may not be blank.')]

            except ValueError:
                errors[key] = [_('This field must be a float.')]

        if errors:
            raise ValidationError(errors)



from django.conf import settings
from django.urls import include, re_path
from django.views.generic import TemplateView

from daiquiri.core.utils import import_class
from daiquiri.jobs.routers import JobRouter

app_name = 'uws'

router = JobRouter(trailing_slash=False)

'''
add uws routes for the resources specified in the settings
resources need to be of the form
{
    'prefix': r'query',
    'viewset': 'daiquiri.query.viewsets.UWSQueryJobViewSet',
    'base_name': 'uws_query'
}
'''
try:
    resources = settings.UWS_RESOURCES
except AttributeError:
    pass
else:
    for resource in resources:
        router.register(resource['prefix'], import_class(resource['viewset']), basename=resource['base_name'])

urlpatterns = [
    re_path(r'^$', TemplateView.as_view(template_name='uws/root.html'), name='uws_root'),
    re_path(r'', include(router.urls)),
]



import csv
import re

from django.contrib.auth.models import User
from django.core.management.base import BaseCommand

from allauth.account.models import EmailAddress

from daiquiri.auth.models import Profile


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("--id", default=".*", help="find users by id")
        parser.add_argument("--email", default=".*", help="find users by email")
        parser.add_argument("--username", default=".*", help="find users by username")
        parser.add_argument(
            "--first_name", default=".*", help="find users by first name"
        )
        parser.add_argument("--last_name", default=".*", help="find users by last name")
        parser.add_argument(
            "--email_verified",
            default=".*",
            help="filter users by email_verified, requires argument, supports: true/false/1/0",
        )
        parser.add_argument(
            "--profile_pending",
            default=".*",
            help="filter users by profile_pending, requires argument, supports: true/false/1/0",
        )
        parser.add_argument(
            "--profile_confirmed",
            default=".*",
            help="filter users by profile_confirmed, requires argument, supports: true/false/1/0",
        )
        parser.add_argument(
            "-p",
            "--print",
            action="store_true",
            help="print found users, don't save them to csv",
        )
        parser.add_argument(
            "-o",
            "--output_file",
            default="found_users.csv",
            help="output file, default is 'found_users.csv'",
        )

    def save_csv(self, data, filename):
        if len(data) > 1:
            data_file = open(filename, "w", newline="", encoding="utf-8")
            csv_writer = csv.DictWriter(
                data_file, fieldnames=list(data[0].keys()), dialect="unix"
            )
            csv_writer.writeheader()
            for user in data:
                csv_writer.writerow(user)
            print("List written to " + filename)

    def print_file(self, filename):
        f = open(filename)
        content = f.read()
        print(content)
        f.close()

    def get_profile(self, username):
        return Profile.objects.get(user__username=username)

    def rx_match(self, regex, s):
        return bool(re.search(regex, str(s)))

    def make_bool(self, val):
        if str(val) == "1":
            return True
        if str(val).lower() == "true":
            return True
        return False

    def bools_are_equal(self, b1, b2):
        return self.make_bool(b1) == self.make_bool(b2)

    def check_match(self, user, profile, email_verified, options):
        if self.rx_match(options["id"], user.id) is False:
            return False
        if self.rx_match(options["email"], user.email) is False:
            return False
        if self.rx_match(options["username"], user.username) is False:
            return False
        if self.rx_match(options["last_name"], user.last_name) is False:
            return False
        if (
            self.bools_are_equal(options["profile_pending"], profile.is_pending)
            is False
        ):
            return False
        if (
            self.bools_are_equal(options["profile_confirmed"], profile.is_confirmed)
            is False
        ):
            return False
        if self.bools_are_equal(options["email_verified"], email_verified) is False:
            return False
        return True

    def find_users(self, options):
        found_users = []
        for _, user in enumerate(User.objects.all().order_by("date_joined")):
            profile = Profile.objects.get(user__username=user.username)
            email = EmailAddress.objects.filter(user=user.id, verified=True).first()
            email_verified = False
            if email is not None:
                email_verified = True
            m = self.check_match(user, profile, email_verified, options)
            if m is True:
                found_users.append(
                    {
                        "id": user.id,
                        "username": user.username,
                        "first_name": user.first_name,
                        "last_name": user.last_name,
                        "date_joined": user.date_joined,
                        "unix_joined": user.date_joined.timestamp(),
                        "email": user.email,
                        "email_verified": email_verified,
                        "last_login": user.last_login,
                        "profile_pending": profile.is_pending,
                        "profile_confirmed": profile.is_confirmed,
                    }
                )
        return found_users

    def handle(self, *args, **options):
        no_total_users = User.objects.all().count()
        print("Total no of users:    %d" % (no_total_users))
        found_users = self.find_users(options)

        print(
            "Matching the filter:  %d  %.2f%%"
            % (len(found_users), (100 / no_total_users) * len(found_users))
        )

        self.save_csv(found_users, options["output_file"])
        if options["print"] is True:
            self.print_file(options["output_file"])



import logging

from django.conf import settings
from django.http import HttpResponseRedirect
from django.urls import reverse

from allauth.account.adapter import DefaultAccountAdapter
from allauth.exceptions import ImmediateHttpResponse
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter

logger = logging.getLogger(__name__)


class DaiquiriAccountAdapter(DefaultAccountAdapter):

    def is_safe_url(self, url):
        from django.utils.http import url_has_allowed_host_and_scheme
        return url_has_allowed_host_and_scheme(url, allowed_hosts=settings.ALLOWED_HOSTS)

    def save_user(self, request, user, form, commit=True):
        super().save_user(request, user, form)

        if settings.AUTH_WORKFLOW:
            user.save()
            user.profile.is_pending = True
            user.profile.save()

    def login(self, request, user):
        if settings.AUTH_WORKFLOW and user.profile.is_pending:
            raise ImmediateHttpResponse(HttpResponseRedirect(reverse('account_pending')))
        else:
            super().login(request, user)

    def is_open_for_signup(self, request):
        return settings.AUTH_SIGNUP


class DaiquiriSocialAccountAdapter(DefaultSocialAccountAdapter):

    def authentication_error(self, request, provider_id, error=None, exception=None, extra_context=None):
        logger.error([provider_id, error, exception])
        super().authentication_error(request, provider_id, error, exception, extra_context)



from rest_framework.filters import BaseFilterBackend

import iso8601

from daiquiri.core.utils import make_query_dict_upper_case
from daiquiri.jobs.models import Job


class UWSFilterBackend(BaseFilterBackend):

    def filter_queryset(self, request, queryset, view):
        query_dict = make_query_dict_upper_case(request.GET)

        # apply only for list
        if view.action == 'list':
            phases = query_dict.getlist('PHASE')

            if phases:
                queryset = queryset.filter(phase__in=phases)
            else:
                queryset = queryset.exclude(phase__exact=Job.PHASE_ARCHIVED)

            after = query_dict.get('AFTER')
            if after:
                queryset = queryset.filter(creation_time__gt=iso8601.parse_date(after))

            last = query_dict.get('LAST')
            if last:
                queryset = queryset.filter(start_time__isnull=False) \
                    .order_by('-start_time')[:int(last)]

        return queryset



import logging
import os
from pathlib import Path
from typing import Union
from urllib.parse import urljoin

from django.conf import settings
from django.shortcuts import render
from django.urls import reverse
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe
from django.utils.timezone import now
from django_sendfile import sendfile

from daiquiri.core.utils import get_client_ip, markdown
from daiquiri.stats.models import Record

from .models import Directory

logger = logging.getLogger(__name__)


def file_exists(file_path: str) -> bool:
    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)
    return os.path.isfile(absolute_file_path)


def get_file_path(file_path: str) -> Union[Path, str, None]:
    if file_exists(file_path):
        return file_path
    elif not file_path or file_path.endswith('/'):
        # try different paths
        for path in [
            file_path.rstrip('/') + '.html',
            file_path.rstrip('/') + '.md',
            file_path + 'index.html',
            file_path + 'index.md',
        ]:
            if file_exists(path):
                return path

    # return None if no file was found
    return None


def get_directory(user, file_path):
    # loop over all directories beginning with the highest depth and return as soon as a directory matches
    for directory in Directory.objects.order_by('-depth'):
        if os.path.normpath(file_path).startswith(directory.path):
            try:
                return Directory.objects.filter_by_access_level(user).get(
                    pk=directory.pk
                )
            except Directory.DoesNotExist:
                return None


def check_file(user, file_path):
    return get_directory(user, file_path) is not None


def render_with_layout(request, file_path):
    context = {}
    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)
    content = read_file_content(absolute_file_path)
    if content:
        context['content'] = content

    return render(request, 'files/layout.html', context)


def read_file_content(abs_file_path):
    """Reads the content of a html- or md-file and returns html"""
    if abs_file_path.endswith('.html') or abs_file_path.endswith('.md'):
        with open(abs_file_path) as f:
            file_content = f.read()

            if abs_file_path.endswith('.html'):
                return mark_safe(file_content)
            elif abs_file_path.endswith('.md'):
                return mark_safe(force_str(markdown(file_content)))
    else:
        return ''


def send_file(request, file_path, search=None):
    # create a stats record for this download
    resource = {'file_path': file_path}
    if search:
        resource['search'] = search

    absolute_file_path = os.path.join(settings.FILES_BASE_PATH, file_path)

    Record.objects.create(
        time=now(),
        resource_type='FILE',
        resource=resource,
        client_ip=get_client_ip(request),
        user=request.user if request.user.is_authenticated else None,
        size=os.path.getsize(absolute_file_path),
    )

    # send the file to the client
    return sendfile(request, absolute_file_path)


def get_url_from_file_path(file_path: Union[Path, str]) -> str:
    url = urljoin(
        settings.FILES_BASE_URL,
        reverse(
            'files:file',
            kwargs={'file_path': file_path},
        ),
    )

    return url


def make_file_path_absolute(file_path: Union[Path, str]) -> Path:
    res_path = Path(settings.FILES_BASE_PATH) / Path(file_path)
    return res_path


def make_file_path_relative(file_path: Union[Path, str]) -> Path:
    res_path = Path(file_path).relative_to(settings.FILES_BASE_PATH)
    return res_path



from django.urls import include, path, re_path

from rest_framework import routers

from .views import ManagementView, SchemasView, SchemaView, TableView
from .viewsets import (
                       AccessLevelViewSet,
                       ColumnViewSet,
                       FunctionViewSet,
                       LicenseViewSet,
                       MetaViewSet,
                       SchemaViewSet,
                       TableViewSet,
)

app_name = 'metadata'

router = routers.DefaultRouter()
router.register(r'schemas', SchemaViewSet, basename='schema')
router.register(r'tables', TableViewSet, basename='table')
router.register(r'columns', ColumnViewSet, basename='column')
router.register(r'functions', FunctionViewSet, basename='function')
router.register(r'licenses', LicenseViewSet, basename='license')
router.register(r'accesslevels', AccessLevelViewSet, basename='accesslevel')
router.register(r'meta', MetaViewSet, basename='meta')

urlpatterns = [
    path('api/', include(router.urls)),

    path('management/', ManagementView.as_view(), name='management'),

    re_path(r'^(?P<schema_name>\w+)/$', SchemaView.as_view(), name='schema'),
    re_path(r'^(?P<schema_name>\w+)/(?P<table_name>\w+)/$', TableView.as_view(), name='table'),
    path(r'', SchemasView.as_view(), name='schemas'),
]



from rest_framework import serializers

from .utils import make_query_dict_upper_case


class JSONDictField(serializers.JSONField):

    initial = dict

    def to_internal_value(self, data):
        data = super().to_internal_value(data)

        if not isinstance(data, dict):
            self.fail('invalid')

        return data


class JSONListField(serializers.JSONField):

    initial = list

    def to_internal_value(self, data):
        data = super().to_internal_value(data)

        if not isinstance(data, list):
            self.fail('invalid')

        return data


class ChoicesSerializer(serializers.Serializer):
    id = serializers.SerializerMethodField()
    text = serializers.SerializerMethodField()

    def get_id(self, obj):
        return obj[0]

    def get_text(self, obj):
        return obj[1]


class CaseInsensitiveSerializer(serializers.Serializer):

    def __init__(self, *args, **kwargs):
        # make all the keys in kwargs['data'] upper case
        if 'data' in kwargs:
            kwargs['data'] = make_query_dict_upper_case(kwargs['data'])

        super().__init__(*args, **kwargs)



import os
from email.utils import getaddresses
from urllib.parse import urlparse

import dj_database_url


def get(key, default=None):
    return os.getenv(key, default)


def get_bool(key, default=None):
    value = os.getenv(key)
    if value:
        return True if value.lower() in ['1', 't', 'true', 'y', 'yes', 'on'] else False
    else:
        return default

def get_abspath(key, default=None):
    value = os.getenv(key, default)
    return os.path.abspath(value) if value else default


def get_schema(key):
    return os.getenv(key, key.lower())


def get_list(key, default=[]):
    value = os.getenv(key)
    if value:
        return [value.strip() for value in value.split(',')]
    else:
        return default


def get_email_list(key, default=[]):
    return getaddresses(get_list(key, default))


def get_url(key, default=None):
    value = os.getenv(key, default)
    return value if value.endswith('/') else value + '/'


def get_database(key):
    database_string = os.getenv(f'DATABASE_{key.upper()}')
    if database_string:
        database_type = urlparse(database_string).scheme

        # rewrite mariadb since it is not supported by dj_database_url
        if database_type == 'mariadb':
            database_string = database_string.replace('mariadb://', 'mysql://')

        database_config = dj_database_url.parse(database_string)

        # patch bug in dj_database_url
        if database_type in ['postgres', 'postgresql', 'pgsql']:
            database_config['ENGINE'] = 'django.db.backends.postgresql'

        return database_config

    else:
        return {}


def get_database_adapter():
    database_string = os.getenv('DATABASE_DATA')
    database_type = urlparse(database_string).scheme

    if database_type in ['postgres', 'postgresql', 'pgsql']:
        return 'daiquiri.core.adapter.database.postgres.PostgreSQLAdapter'
    elif database_type == 'mysql':
        return 'daiquiri.core.adapter.database.mysql.MySQLAdapter'
    elif database_type == 'mariadb':
        return 'daiquiri.core.adapter.database.mariadb.MariaDBAdapter'
    else:
        return None


def get_download_adapter():
    database_string = os.getenv('DATABASE_DATA')
    database_type = urlparse(database_string).scheme

    if database_type in ['postgres', 'postgresql', 'pgsql']:
        return 'daiquiri.core.adapter.download.pgdump.PgDumpAdapter'
    elif database_type in ['mysql', 'mariadb']:
        return 'daiquiri.core.adapter.download.mysqldump.MysqldumpAdapter'
    else:
        return None



from django.conf import settings
from django.urls import reverse

from daiquiri.core.utils import sanitize_str

testset_login = {
    "url": "account_login",
    "data": {
        "login": "user",
        "password": "user",
    },
}


testset_signup = {
    "url": "account_signup",
    "data": {
        "email": "testing@example.com",
        "username": "testing",
        "first_name": "Tanja",
        "last_name": "Test",
        "password1": "testing",
        "password2": "testing"
    },
}

testset_contact = {
    "url": "contact:contact",
    "data": {"subject": "Test", "message": "This is a test."},
}


def run_honeypot_test(db, client, testset, honeypot, exp):
    data = testset["data"]
    url = reverse(testset["url"])
    if settings.HONEYPOT_ENABLED is False:
        exp = 302
        if "contact" in url:
            exp = 200
    if honeypot is not None:
        data[sanitize_str(settings.HONEYPOT_FIELD_NAME)] = honeypot

    response = client.post(url, data)
    assert response.status_code == exp


def test_contact_post(db, client):
    client.login(username="admin", password="admin")
    run_honeypot_test(db, client, testset_contact, settings.HONEYPOT_FIELD_VALUE, 200)


def test_contact_post_honeypot_field_invalid(db, client):
    client.login(username="admin", password="admin")
    run_honeypot_test(db, client, testset_contact, "some_text", 400)


def test_contact_post_honeypot_field_missing(db, client):
    client.login(username="admin", password="admin")
    run_honeypot_test(db, client, testset_contact, None, 400)



# Generated by Django 2.1.3 on 2018-11-05 16:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_stats', '0004_set_record_null'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='record',
            options={'ordering': ('-time',), 'verbose_name': 'Record', 'verbose_name_plural': 'Records'},
        ),
    ]



import pytest

from django.contrib.auth.models import Group
from django.urls import reverse

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'detail': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'create': {
        'admin': 405, 'manager': 403, 'user': 403, 'anonymous': 403
    },
    'update': {
        'admin': 405, 'manager': 403, 'user': 403, 'anonymous': 403
    },
    'delete': {
        'admin': 405, 'manager': 403, 'user': 403, 'anonymous': 403
    }
}

urlnames = {
    'list': 'auth:group-list',
    'detail': 'auth:group-detail'
}

instances = [10]


@pytest.mark.parametrize(('username', 'password'), users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        assert len(response.json()) == 7


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_detail(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Group.objects.get(pk=pk)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.get(url)
    assert response.status_code == status_map['detail'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('name') == instance.name


@pytest.mark.parametrize(('username', 'password'), users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {})
    assert response.status_code == status_map['create'][username], response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_update(db, client, username, password, pk):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.put(url, {}, content_type='application/json')
    assert response.status_code == status_map['update'][username], response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_delete(db, client, username, password, pk):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.delete(url)
    assert response.status_code == status_map['delete'][username], response.json()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-18 17:10
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0017_rename_database_to_schema'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='column',
            options={'ordering': ('table__schema__order', 'table__order', 'order'), 'permissions': (('view_column', 'Can view Column'),), 'verbose_name': 'Column', 'verbose_name_plural': 'Columns'},
        ),
    ]
